/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlEndMachineDialog.h
//
// @short    See nlEndMachineDialog.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlModel.h>
#include <nlCompositeHandle.h>
#include <nlCommonType.h>

//2 From SystemMgr
#include <nlClientInterface.h>
#include <nlSystemMgrType.h>
#include <nlCommand.h>

//1 Local
#include "Main.h"
#include "nlEndMachineDlg.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  const CString STR_ON = _T( "On" );
  const CString STR_OFF = _T( "Off" );
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlEndMachineDialog::SnlImp* CnlEndMachineDialog::Imp()       { return m_pImp; }
inline const  CnlEndMachineDialog::SnlImp* CnlEndMachineDialog::Imp() const { return m_pImp; }

struct CnlEndMachineDialog::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlEndMachineDialog* const pOwner,
    CnlClientInterface* const pClientInterface,
    CnlCompositeHandle hEndMachine );

  // Destructor
   ~SnlImp();

  // OK or Cancel
  void Close();

  // Refresh the IP address assigned to the NIC or selected port
  void UpdateIpAddress();

  // Refresh the MAC address assigned to the NIC or selected port
  void UpdateMacAddress();

  // Refresh the status of the power button
  void UpdatePowerStatus();

  // Enable or disable the dialog
  void EnableDialog(
    const bool bEnable );

  // Enable or disable the command buttons on this dialog
  void EnableCommandButtons(
    const bool bEnable );

  // Check the EnlResult returned and display if an error occurred
  void CheckResult(
    const NnlSystemMgrType::EnlResult eResult );

  // Is this machine on?
  bool IsOn() const;
    
  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlEndMachineDialog* const m_pOwner;

  // Handle to the model
  CnlClientInterface* m_pClientInterface;

  // Handle to the Machine I am controlling
  const CnlCompositeHandle m_hMachine;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // Desc

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlEndMachineDialog::SnlImp::SnlImp(
  CnlEndMachineDialog* const pOwner,
  CnlClientInterface* const pClientInterface,
  CnlCompositeHandle hEndMachine )
 :
  m_pOwner( pOwner ),
  m_pClientInterface( pClientInterface ),
  m_hMachine( hEndMachine )
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::SnlImp::Close()
{
  m_pClientInterface->DetachObserver( m_pOwner );
  m_pOwner->CDialog::OnCancel();
}


///////////////////////////////////////////////////////////////////////////////
CnlEndMachineDialog::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::SnlImp::UpdateIpAddress()
{
  //1 Identify which port is selected
  CComboBox* pPortCombo = 
    static_cast< CComboBox* >( m_pOwner->GetDlgItem( IDC_COMBO_NIC ) );
  const unsigned int nPortSelected = pPortCombo->GetCurSel();

  //1 Get the associated IP Address
  CStatic* pIpAddressValue =
    static_cast< CStatic* >( m_pOwner->GetDlgItem( IDC_STATIC_IP_ADDRESS ) );
  CString strIpAddress;
  NnlSystemMgrType::EnlResult eResult = 
    m_pClientInterface->GetProcessingMachineIpAddress( 
      strIpAddress, 
      m_hMachine,
      nPortSelected );  // NIC 0
  assert( eResult = NnlSystemMgrType::RESULT_OK );

  //1 Display it
  pIpAddressValue->SetWindowText( strIpAddress );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::SnlImp::UpdateMacAddress()
{
  //1 Identify the hardware to display info for
  CComboBox* pPortCombo = 
    static_cast< CComboBox* >( m_pOwner->GetDlgItem( IDC_COMBO_NIC ) );
  const unsigned int nPortSelected = pPortCombo->GetCurSel();

  //1 Get the corresponding MAC Address
  CStatic* pMacAddressValue =
    static_cast< CStatic* >( m_pOwner->GetDlgItem( IDC_STATIC_MAC_ADDRESS ) );
  CString strMacAddress;
  NnlSystemMgrType::EnlResult eResult = 
    m_pClientInterface->GetProcessingMachineMacAddress( 
      strMacAddress, 
      m_hMachine,
      nPortSelected );  // NIC 0
  assert( eResult = NnlSystemMgrType::RESULT_OK );

  //1 Display it
  pMacAddressValue->SetWindowText( strMacAddress );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::SnlImp::UpdatePowerStatus()
{
  //1 Get the on/off power status of this machine
  bool bPowerStatus;
  NnlSystemMgrType::EnlResult eResult = 
    m_pClientInterface->GetProcessingMachinePowerStatus( 
      bPowerStatus, 
      m_hMachine );
  assert( eResult = NnlSystemMgrType::RESULT_OK );

  //1 Display it
  EnableDialog( bPowerStatus );
  const CString strPowerStatus = bPowerStatus ? STR_ON : STR_OFF;
  CButton* pPowerButton = 
    static_cast< CButton* >( m_pOwner->GetDlgItem( IDC_BUTTON_POWER ) );
  pPowerButton->SetWindowText( strPowerStatus );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::SnlImp::EnableDialog(
  const bool bEnable )
{
  CWnd* pWnd = NULL;

  EnableCommandButtons( bEnable );
  
  pWnd = (CWnd *) m_pOwner->GetDlgItem( IDC_COMBO_NIC );
  pWnd->EnableWindow( bEnable );

  pWnd = (CWnd *) m_pOwner->GetDlgItem( IDC_EDIT_IP_ADDRESS );
  pWnd->EnableWindow( bEnable );

  pWnd = (CWnd *) m_pOwner->GetDlgItem( IDC_BUTTON_ARP_A );
  pWnd->EnableWindow( bEnable );

  pWnd = (CWnd *) m_pOwner->GetDlgItem( IDC_BUTTON_NETSTAT_RN );
  pWnd->EnableWindow( bEnable );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::SnlImp::EnableCommandButtons(
  const bool bEnable )
{
  CWnd* pWnd = NULL;

  pWnd = (CWnd *) m_pOwner->GetDlgItem( IDC_BUTTON_ARP_D );
  pWnd->EnableWindow( bEnable );

  pWnd = (CWnd *) m_pOwner->GetDlgItem( IDC_BUTTON_ASSIGN_IP );
  pWnd->EnableWindow( bEnable );

  pWnd = (CWnd *) m_pOwner->GetDlgItem( IDC_BUTTON_SEND_ARP );
  pWnd->EnableWindow( bEnable );

  pWnd = (CWnd *) m_pOwner->GetDlgItem( IDC_BUTTON_SEND_PING );
  pWnd->EnableWindow( bEnable );

}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::SnlImp::CheckResult(
  const NnlSystemMgrType::EnlResult eResult )
{
  if ( eResult != NnlSystemMgrType::RESULT_OK )
  {
    const CString strResult = m_pClientInterface->ResultToString( eResult );
    AfxMessageBox( strResult );
  }
}

///////////////////////////////////////////////////////////////////////////////
bool CnlEndMachineDialog::SnlImp::IsOn() const
{
  CButton* pPowerButton = 
    static_cast< CButton* >( m_pOwner->GetDlgItem( IDC_BUTTON_POWER ) );
  TCHAR lpszBuf[10];
  pPowerButton->GetWindowText( lpszBuf, 10 );
  const CString strStatus = lpszBuf;
  return ( strStatus == STR_ON );
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlEndMachineDialog::CnlEndMachineDialog(
  CnlClientInterface* const pClientInterface,
  CnlCompositeHandle hEndMachine,
  CWnd* pParent  /*=NULL*/ )
 :
  CDialog(CnlEndMachineDialog::IDD, pParent)
{
  //{{AFX_DATA_INIT(CnlEndMachineDialog)
    // NOTE: the ClassWizard will add member initialization here
  //}}AFX_DATA_INIT
  m_pImp = new SnlImp( this, pClientInterface, hEndMachine );
}

///////////////////////////////////////////////////////////////////////////////
CnlEndMachineDialog::~CnlEndMachineDialog()
{
  delete m_pImp;
}


void CnlEndMachineDialog::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CnlEndMachineDialog)
    // NOTE: the ClassWizard will add DDX and DDV calls here
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CnlEndMachineDialog, CDialog)
  //{{AFX_MSG_MAP(CnlEndMachineDialog)
  ON_BN_CLICKED(IDC_BUTTON_SEND_ARP, OnButtonSendArp)
  ON_BN_CLICKED(IDC_BUTTON_ARP_A, OnButtonArpA)
  ON_BN_CLICKED(IDC_BUTTON_SEND_PING, OnButtonSendPing)
  ON_BN_CLICKED(IDC_BUTTON_NETSTAT_RN, OnButtonNetstatRn)
  ON_CBN_SELCHANGE(IDC_COMBO_NIC, OnSelchangeComboPort)
  ON_BN_CLICKED(IDC_BUTTON_POWER, OnButtonPower)
  ON_BN_CLICKED(IDC_BUTTON_ASSIGN_IP, OnButtonAssignIp)
  ON_BN_CLICKED(IDC_BUTTON_ARP_D, OnButtonArpD)
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////
BOOL CnlEndMachineDialog::OnInitDialog()
{
  CDialog::OnInitDialog();

  //1 Register to receive updates
  Imp()->m_pClientInterface->AttachObserver( this );

  //1 See if I am turne don
  Imp()->UpdatePowerStatus();

  if ( ! Imp()->IsOn() )
  {
    return TRUE;
  }

  //1 Handle machine-specifics
  CStatic* pNicPort = 
    static_cast< CStatic* >( GetDlgItem( IDC_STATIC_NIC_Port ) );
  CButton* pAssignButton = 
    static_cast< CButton* >( GetDlgItem( IDC_BUTTON_ASSIGN_IP ) );

  if ( Imp()->m_hMachine.Type() == NnlType::HANDLE_TYPE_END_MACHINE )
  {
    pNicPort->SetWindowText( _T( "NIC:" ) );
    pAssignButton->SetWindowText( _T( "Assign to NIC" ) );
    
  }
  else
  {
    pNicPort->SetWindowText( _T( "Port:" ) );
    pAssignButton->SetWindowText( _T( "Assign to Port" ) );
  }
  
  //1 Handle the port combo box
  CComboBox* pPortCombo = 
    static_cast< CComboBox* >( GetDlgItem( IDC_COMBO_NIC ) );

  unsigned int nNumNics;
  NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->GetProcessingMachineNicCount(
      nNumNics,
      Imp()->m_hMachine );

  for ( unsigned int nNic = 0; nNic < nNumNics; ++nNic )
  {
    CString strNic;
    strNic.Format( _T( "%d" ), nNic );
    pPortCombo->AddString( strNic);
  }

  pPortCombo->SetCurSel( 0 );

  //1 Do an update
  Imp()->UpdateIpAddress();
  Imp()->UpdateMacAddress();

  const bool bCommandExecuting = 
    Imp()->m_pClientInterface->IsCommandExecuting();

  Imp()->EnableCommandButtons( ! bCommandExecuting );

  return TRUE;  // return TRUE unless you set the focus to a control
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnOK() 
{
  //1 Having this prevents <enter> from closing the dialog
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnCancel() 
{
  Imp()->Close();
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::Update(
  CnlModel* const pModel,
  const int eChangeType )
{
  NnlSystemMgrType::EnlMessage eMessage = 
    static_cast< NnlSystemMgrType::EnlMessage >( eChangeType );

  switch( eMessage )
  {
    case NnlSystemMgrType::MESSAGE_IP_ADDRESS_CHANGED :
    {
      Imp()->UpdateIpAddress();
      break;
    }
    case NnlSystemMgrType::MESSAGE_POWER_STATUS_CHANGED :
    {
      Imp()->UpdatePowerStatus();
      break;
    }
    case NnlSystemMgrType::MESSAGE_COMMAND_STARTED :
    {
      Imp()->EnableCommandButtons( false );
      break;
    }
    case NnlSystemMgrType::MESSAGE_COMMAND_COMPLETED :
    {
      Imp()->EnableCommandButtons( true );
      break;
    }
    
    default : break;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::StopObserving(
  CnlModel* const pModel )
{
  if ( pModel == reinterpret_cast< CnlModel* >( Imp()->m_pClientInterface ) )
  {
    Imp()->m_pClientInterface = NULL;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnButtonSendArp() 
{
  //1 Identify the "Who has" machine
  CEdit* pDebugCommandEdit = (CEdit *) GetDlgItem( IDC_EDIT_IP_ADDRESS );
  TCHAR lpszBuf[100];
  pDebugCommandEdit->GetWindowText( lpszBuf, 100 );
  const CString strIpAddress = lpszBuf;

  //1 Send the command
  const CnlCommand* const arp = Imp()->m_pClientInterface->MakeCommandArp(
    strIpAddress );
  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->ExecuteCommand(
      Imp()->m_hMachine,
      arp );

  //1 Close the dialog so the user can follow the progress
  Imp()->Close();
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnButtonSendPing()
{
  //1 Identify the "Ping target" machine
  CEdit* pDebugCommandEdit = (CEdit *) GetDlgItem( IDC_EDIT_IP_ADDRESS );
  TCHAR lpszBuf[100];
  pDebugCommandEdit->GetWindowText( lpszBuf, 100 );
  const CString strPingTarget = lpszBuf;

  //1 Send the command
  const CnlCommand* const ping = Imp()->m_pClientInterface->MakeCommandPing(
    strPingTarget );
  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->ExecuteCommand(
      Imp()->m_hMachine,
      ping );

  //1 Close the dialog so the user can follow the progress
  Imp()->Close();
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnButtonArpA() 
{
  //1 Request the arp cache string
  CString strArpCache;
  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->GetProcessingMachineString( 
      strArpCache, 
      Imp()->m_hMachine,
      NnlSystemMgrType::COMMAND_LINE_ENTRY_ARP_A );

  //1 Display it
  AfxMessageBox( strArpCache );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnButtonArpD() 
{
  //1 Tell the model to delete the cache
  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->ClearProcessingMachineArpCache( 
      Imp()->m_hMachine );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnButtonNetstatRn() 
{
  //1 Request the routing table string
  CString strNetstat;
  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->GetProcessingMachineString( 
      strNetstat, 
      Imp()->m_hMachine,
      NnlSystemMgrType::COMMAND_LINE_ENTRY_NETSTAT_RN );

  //1 Display it
  AfxMessageBox( strNetstat );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnSelchangeComboPort() 
{
  Imp()->UpdateIpAddress();
  Imp()->UpdateMacAddress();
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnButtonPower() 
{
  //1 Get the current setting
  CButton* pPowerButton = 
    static_cast< CButton* >( GetDlgItem( IDC_BUTTON_POWER ) );
  const bool bPowerStatus = ( pPowerButton->GetCheck() == 1 );

  //1 Request a toggle
  NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->ToggleProcessingMachinePower( 
      Imp()->m_hMachine );
  assert( eResult = NnlSystemMgrType::RESULT_OK );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndMachineDialog::OnButtonAssignIp() 
{
  //1 Identify the IP address to assign
  CEdit* pDebugCommandEdit = (CEdit *) GetDlgItem( IDC_EDIT_IP_ADDRESS );
  TCHAR lpszBuf[100];
  pDebugCommandEdit->GetWindowText( lpszBuf, 100 );
  const CString strIpAddress = lpszBuf;
  unsigned int nPortOffset = 0;

  //1 For routers, get the port#
  if ( Imp()->m_hMachine.Type() == NnlType::HANDLE_TYPE_ROUTER )
  {
    //3 Get the port# from the combo box
    CComboBox* pPortCombo = 
      static_cast< CComboBox* >( GetDlgItem( IDC_COMBO_NIC ) );
    nPortOffset = pPortCombo->GetCurSel();
  }

  //1 Request assignment
  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->AssignIpAddress(
      Imp()->m_hMachine,
      nPortOffset,
      strIpAddress );

  //1 Verify result
  Imp()->CheckResult( eResult );
}

