/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlCompositeHandle.h
//
// @short    See nlCompositeHandle.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From 

//1 Local
#include "nlCompositeHandle.h"
#include "nlCommonType.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlCompositeHandle::SnlImp* CnlCompositeHandle::Imp()       { return m_pImp; }
inline const  CnlCompositeHandle::SnlImp* CnlCompositeHandle::Imp() const { return m_pImp; }

struct CnlCompositeHandle::SnlImp
{
  //1 Function Prototypes
  
  // Constructor for top-level handles
  SnlImp(
    CnlCompositeHandle* const pOwner,
    const EnlHandleType eType,
    const unsigned long nID );

  // Constructor for nested handles
  SnlImp(
    CnlCompositeHandle* const pOwner,
    const EnlHandleType eType,
    const CnlCompositeHandle hComposite,
    const unsigned long nID );

  // Copy Constructor
  SnlImp(
    CnlCompositeHandle* const pOwner,
    const SnlImp& rhs );

  // Destructor
   ~SnlImp();

  // Equivalence testing
  bool operator==(
    const SnlImp& rhs ) const;

  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CnlCompositeHandle* const m_pOwner;

  // Handle to the composite object, if any, that contains the object I refer to
  const CnlCompositeHandle* const m_pComposite;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The type of object I point to
  const EnlHandleType m_eType;

  // The number that uniquely identifies this object
  const unsigned long m_nID;

  // For Debugging
  CString m_strDescr;
  
private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::SnlImp::SnlImp(
  CnlCompositeHandle* const pOwner,
  const EnlHandleType eType,
  const unsigned long nID )
 :
  m_pOwner( pOwner ),
  m_eType( eType ),
  m_pComposite( NULL ),
  m_nID( nID )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::SnlImp::SnlImp(
  CnlCompositeHandle* const pOwner,
  const EnlHandleType eType,
  const CnlCompositeHandle hComposite,
  const unsigned long nID )
 :
  m_pOwner( pOwner ),
  m_eType( eType ),
  m_pComposite( new CnlCompositeHandle( hComposite ) ),
  m_nID( nID )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::SnlImp::SnlImp(
  CnlCompositeHandle* const pOwner,
  const SnlImp& rhs )
 :
  m_pOwner( pOwner ),
  m_eType( rhs.m_eType ),
  m_pComposite( rhs.m_pComposite == NULL ? 
                NULL : new CnlCompositeHandle( * ( rhs.m_pComposite ) ) ),
  m_nID( rhs.m_nID )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::SnlImp::~SnlImp()
{
  delete m_pComposite;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlCompositeHandle::SnlImp::operator==(
  const SnlImp& rhs ) const
{
  bool bEqual = true;

  bEqual &= ( m_eType == rhs.m_eType );
  
  if ( ( m_pComposite == NULL ) || ( rhs.m_pComposite == NULL ) )
  {
    // We would both have to be NULL to be equal
    bEqual &= ( m_pComposite == rhs.m_pComposite );
  }
  else
  {
    bEqual &= ( * m_pComposite == * rhs.m_pComposite );
  }

  bEqual &= ( m_nID == rhs.m_nID );

  return bEqual;
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::CnlCompositeHandle()
{
  m_pImp = new SnlImp( this, HANDLE_TYPE_UNKNOWN, 0 );

  Imp()->m_strDescr = Descr();
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::CnlCompositeHandle(
  const EnlHandleType eType,
  const unsigned long nID )
{
  assert( nID != 0 );
  m_pImp = new SnlImp( this, eType, nID );
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::CnlCompositeHandle(
  const EnlHandleType eType,
  const CnlCompositeHandle hComposite,
  const unsigned long nID )
{
  assert( nID != 0 );
  m_pImp = new SnlImp( this, eType, hComposite, nID );
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::CnlCompositeHandle( 
  const CnlCompositeHandle& rhs )
{
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle::~CnlCompositeHandle()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle& CnlCompositeHandle::operator=( 
  const CnlCompositeHandle& rhs )
{
  delete m_pImp;
  m_pImp = new SnlImp( this, * (rhs.m_pImp ) );
  return *this;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlCompositeHandle::operator==(
  const CnlCompositeHandle& rhs ) const
{
  return ( *m_pImp == * ( rhs.m_pImp ) );
}

///////////////////////////////////////////////////////////////////////////////
EnlHandleType CnlCompositeHandle::Type() const
{
  return Imp()->m_eType;
}

///////////////////////////////////////////////////////////////////////////////
unsigned long CnlCompositeHandle::ID() const
{
  return Imp()->m_nID;
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle CnlCompositeHandle::Composite() const
{
  if ( Imp()->m_pComposite == NULL )
  {
    // I have no composite handle; return an invalid handle
    return CnlCompositeHandle();
  }

  return ( * Imp()->m_pComposite );
}

///////////////////////////////////////////////////////////////////////////////
bool CnlCompositeHandle::IsValid() const
{
  const bool bValid = 
    (  ( Imp()->m_eType != HANDLE_TYPE_UNKNOWN )
    && (Imp()->m_nID != 0 ) );
  return bValid;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlCompositeHandle::Descr() const
{
  CString strDescr;
  
  switch( Imp()->m_eType )
  {
    case HANDLE_TYPE_UNKNOWN :
    {
      strDescr = _T("Unknown " );
      break;
    }
    case HANDLE_TYPE_END_MACHINE :
    {
      strDescr = _T("EM " );
      break;
    }
    case HANDLE_TYPE_ROUTER :
    {
      strDescr = _T("Router " );
      break;
    }
    case HANDLE_TYPE_NETWORK :
    {
      strDescr = _T("Network " );
      break;
    }
    default : assert( 0 );  // Don't kow this type
  }

  CString strId;
  strId.Format( _T( "%d" ), Imp()->m_nID );

  return strDescr + strId;
}


