//////////////////////////////////////////////////////////////////////////////
//
// @descr    Handle for model objects. Can be used for objects organized by the
//           Composite design pattern.
//
// @short    Composite object handle.
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_COMPOsITE_HANDLE_H__
#define __NL_COMPOsITE_HANDLE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlCommonType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlCompositeHandle
{
  public:
    //1 Final
    // @descr  Default constructor. Creates an invalid handle;
    // @end
    explicit CnlCompositeHandle();

    // @descr  Constructor for handles to top-level objects
    // @param  eType  IN: Type of object I point to
    // @param  nID  IN: The number that uniquely identifies this object in its context.
    //                  If nID == 0, an invaid handle will be created.
    // @end
    explicit CnlCompositeHandle(
      const NnlType::EnlHandleType eType,
      const unsigned long nID );

    // @descr  Constructor for nested objects (those owned by a composite object).
    // @param  eType  IN: Type of object I point to
    // @param  hComposite  IN: Handle to composite object that contains the 
    //                        object I refer to.
    // @param  nID  IN: The number that uniquely identifies this object in its context.
    //                  If nID == 0, an invaid handle will be created.
    // @end
    explicit CnlCompositeHandle(
      const NnlType::EnlHandleType eType,
      const CnlCompositeHandle hComposite,
      const unsigned long nID );

    // @descr  Copy constructor
    // @end
    CnlCompositeHandle( 
      const CnlCompositeHandle& rhs );

    // @descr  Assignment operator
    // @end
    CnlCompositeHandle& operator=( 
      const CnlCompositeHandle& rhs );

    // @descr  Equivalence operator
    // @end
    bool operator==(
      const CnlCompositeHandle& rhs ) const;

    // @descr   Type accessor.
    //          Note: This fn is temporary and should not be used by any views.
    // @return  The method used by this handle to identify the target type.
    // @end
    NnlType::EnlHandleType Type() const;

    // @descr   ID accessor.
    //          Note: This fn is temporary and should not be used by any views.
    // @return  The ID used by this handle to identify End Machines.
    // @end
    unsigned long ID() const;

    // @descr   Composite handle accessor
    // @return  The handle used to identify the owning network.
    // @end
    CnlCompositeHandle Composite() const;

    // @descr   Identifies if the handle refers to a vaid object.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Result returned
    // @return  True if valid; False otherwise
    // @end
    bool IsValid() const;

    // For Debugging: return a descriptive string
    CString Descr() const;
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlCompositeHandle();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlCompositeHandle* operator&();
    const CnlCompositeHandle* operator&() const;
};

#endif

