//////////////////////////////////////////////////////////////////////////////
//
// @descr    A managed pool of unsigned integers. Values can be checked out for use and 
//           returned to make available again. The value 0 is reserved for "invalid".
//
// @short    A managed pool of integers.
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_INTEGER_POOL_H__
#define __NL_INTEGER_POOL_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlIntegerPool
{
  public:
    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlIntegerPool(
      const unsigned long nMaxPoolSize );

    // @descr   Identifies if any values are available for use.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Result returned.
    // @return  True if at least 1 value is available; False otherwise.
    // @end
    bool IsAvailable() const;

    // @descr   Get an unused value, thus reserving it.
    //          Preconditions:
    //            1. At least one value is available.
    //          Postconditions(dependencies):
    //            a. (all) Nonzero value returned; else 0 returned.
    // @return  Value
    // @end
    unsigned long CheckOut();

    // @descr   Return a used value, thus unreserving it.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Value made available for checkout
    // @param   nValue  IN: Value to release.
    // @end
    void CheckIn(
      const unsigned long nValue );
 
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlIntegerPool();

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlIntegerPool( const CnlIntegerPool& );
    CnlIntegerPool& operator=( const CnlIntegerPool& );
    CnlIntegerPool* operator&();
    const CnlIntegerPool* operator&() const;
};

#endif

