/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlVariableParmContainer.h
//
// @short    See nlVariableParmContainer.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From 

//1 Local
#include "nlVariableParmContainer.h"
#include "nlCommonType.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  const int MAX_BOOL_PARMS = 32;

  typedef std::vector< int >::const_iterator NumIter;
  typedef std::vector< CString >::const_iterator StringIter;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlVariableParmContainer::SnlImp* CnlVariableParmContainer::Imp()       { return m_pImp; }
inline const  CnlVariableParmContainer::SnlImp* CnlVariableParmContainer::Imp() const { return m_pImp; }

struct CnlVariableParmContainer::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlVariableParmContainer* const pOwner );

  // Destructor
   ~SnlImp();

  // Copy constructor
  SnlImp( 
    CnlVariableParmContainer* const pOwner,
    const SnlImp& rhs );

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlVariableParmContainer* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The bool parms
  unsigned int m_nNumBoolParameters;
  std::bitset< MAX_BOOL_PARMS > m_Bools;
  
  // The numeric parms
  std::vector< int > m_Numerics;

  // The string parms
  std::vector< CString > m_Strings;

private:

  //1 Unimplemented
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer::SnlImp::SnlImp(
  CnlVariableParmContainer* const pOwner )
 :
  m_pOwner( pOwner ),
  m_nNumBoolParameters( 0 )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer::SnlImp::SnlImp( 
  CnlVariableParmContainer* const pOwner,
  const SnlImp& rhs )
 :
  m_pOwner( pOwner )
{
  m_nNumBoolParameters = rhs.m_nNumBoolParameters;
  m_Bools = rhs.m_Bools;
  std::copy( 
    rhs.m_Numerics.begin(), 
    rhs.m_Numerics.end(), 
    std::inserter( m_Numerics, m_Numerics.begin() ) );
  std::copy( 
    rhs.m_Strings.begin(), 
    rhs.m_Strings.end(), 
    std::inserter( m_Strings, m_Strings.begin() ) );
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer::CnlVariableParmContainer()
{
  m_pImp = new SnlImp( this );
}

///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer::~CnlVariableParmContainer()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer::CnlVariableParmContainer( 
  const CnlVariableParmContainer& rhs )
{
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
}
    
///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer& CnlVariableParmContainer::operator=( 
  const CnlVariableParmContainer& rhs )
{
  delete m_pImp;
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
  return *this;
}

///////////////////////////////////////////////////////////////////////////////
void CnlVariableParmContainer::AddBoolParameter(
  const bool bParameter )
{
  Imp()->m_Bools[ Imp()->m_nNumBoolParameters ] = bParameter;
  ++Imp()->m_nNumBoolParameters;
}

///////////////////////////////////////////////////////////////////////////////
void CnlVariableParmContainer::AddNumericParameter(
  const int nParameter )
{
  Imp()->m_Numerics.push_back( nParameter );
}
    
///////////////////////////////////////////////////////////////////////////////
void CnlVariableParmContainer::AddStringParameter(
  const CString& strParameter )
{
  Imp()->m_Strings.push_back( strParameter );
}
     
///////////////////////////////////////////////////////////////////////////////
unsigned int CnlVariableParmContainer::GetNumBoolParameters()
{
  return Imp()->m_nNumBoolParameters;
}

///////////////////////////////////////////////////////////////////////////////
unsigned int CnlVariableParmContainer::GetNumNumericParameters()
{
  return Imp()->m_Numerics.size();
}
    
///////////////////////////////////////////////////////////////////////////////
unsigned int CnlVariableParmContainer::GetNumStringParameters()
{
  return Imp()->m_Strings.size();
}
    
///////////////////////////////////////////////////////////////////////////////
bool CnlVariableParmContainer::GetBoolParameter(
  const unsigned int nParameterOffset )
{
  if ( nParameterOffset >= Imp()->m_nNumBoolParameters)
  {
    return false;
  }
  
  return Imp()->m_Bools[ nParameterOffset ];
}

///////////////////////////////////////////////////////////////////////////////
int CnlVariableParmContainer::GetNumericParameter(
  const unsigned int nParameterOffset )
{
  if ( nParameterOffset >= Imp()->m_Numerics.size() )
  {
    return 0;
  }
  
  return Imp()->m_Numerics[ nParameterOffset ];
}
    
///////////////////////////////////////////////////////////////////////////////
CString CnlVariableParmContainer::GetStringParameter(
  const unsigned int nParameterOffset )
{
  if ( nParameterOffset >= Imp()->m_Strings.size() )
  {
    return STR_NULL;
  }
  
  return Imp()->m_Strings[ nParameterOffset ];
}

///////////////////////////////////////////////////////////////////////////////
void CnlVariableParmContainer::Dump() const
{
  TRACE0( _T( "\nVPC Dump:" ) );
  
  for ( unsigned int nBool = 0; nBool < Imp()->m_nNumBoolParameters; ++nBool )
  {
    TRACE1( _T( "\nBool %d" ), Imp()->m_Bools[ nBool ] );
  }

  for ( NumIter nIter = Imp()->m_Numerics.begin(); 
        nIter != Imp()->m_Numerics.end();
        ++nIter )
  {
    TRACE1( _T( "\nint %d" ), *nIter );
  }

  for ( StringIter strIter = Imp()->m_Strings.begin(); 
        strIter != Imp()->m_Strings.end();
        ++strIter )
  {
    TRACE1( _T( "\nstring %s" ), *strIter );
  }

}

