//////////////////////////////////////////////////////////////////////////////
//
// @descr    Interface used by "clients" of NetLearn functionality, including 
//           dialogs and the RPI interface
//
// @short    Interface used by "clients"
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_CLIENT_INTERFACE_H__
#define __NL_CLIENT_INTERFACE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From Utilities
#include <nlObserver.h>

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlCompositeHandle;
class CnlEndPayload;
class CnlCommand;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlClientInterface
{
  public:
    //1 Abstract
    // @descr   Attach the specified observer to the models
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Observer attached to all observable models
    // @param   pObserver  IN: The observer to attach
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult AttachObserver(
      CnlObserver* const pObserver ) = 0;

    // @descr   Detach the specified observer from the models
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (all) Observer detached to all observable models
    // @param   pObserver  IN: The observer to detach
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult DetachObserver(
      CnlObserver* const pObserver ) = 0;

    // @descr   Create a subnet and add it to the model.
    //          Preconditions:
    //            1. Valid parms
    //          Postconditions(dependencies):
    //            a. (all) Network created
    // @param   hNetwork  OUT: Handle to the newly-created network
    // @param   strSubnetAddress  IN: Network address in classless form.
    //                            Valid if:
    //                            - well-formed
    //                            - globally unique
    //                            - min 2 bits for end machines
    // @return  True if subnet created; False otherwise
    // @end
    virtual NnlSystemMgrType::EnlResult AddNetworkToInternet(
      CnlCompositeHandle& hNetwork,
      const CString& strSubnetAddress ) = 0;

    // @descr   Add a single end machine to the specified network
    //          Preconditions:
    //            1. The network accepts
    //          Postconditions(dependencies):
    //            a. (none) End machine added
    // @param   hProcessingMachine  OUT: Handle to the end machine created
    // @param   hNetwork  IN: Handle of network to modify
    // @return  True if end machine added; False otherwise
    // @end
    virtual NnlSystemMgrType::EnlResult AddProcessingMachineToNetwork(
      CnlCompositeHandle& hProcessingMachine,
      const CnlCompositeHandle hNetwork ) = 0;    

    // @descr   Create a router
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Router created
    // @param   hRouter  OUT: Handle to the newly created router
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult MakeRouter(
      CnlCompositeHandle& hRouter) = 0;    

    // @descr   Make an Arp command. 
    //          Pass the resulting pointer to ExecuteCommand,
    //          which will delete the object when it is through with it.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Command returned
    // @param   strWhoHasIpAddress  IN: The "Who has" machine address
    // @return  Pointer to the Arp command
    // @end
    virtual const CnlCommand* MakeCommandArp(
      const CString& strWhoHasIpAddress ) const = 0;

    // @descr   Make a Ping command. 
    //          Pass the resulting pointer to ExecuteCommand,
    //          which will delete the object when it is through with it.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Command returned
    // @param   strPingTargetAddress  IN: The machine being pinged
    // @return  Pointer to the Ping command
    // @end
    virtual const CnlCommand* MakeCommandPing(
      const CString& strPingTargetAddress ) const = 0;

    // @descr   Execute a command on the specified end machine.
    //          Preconditions:
    //            1. Machine exists
    //            2. Valid command
    //          Postconditions(dependencies):
    //            a. (all) Command executed
    // @param   hProcessingMachine  IN: Machine to execute on
    // @param   pCommand   IN: Command to execute
    // @param   strParameter  IN: Command parameter
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult ExecuteCommand(
      const CnlCompositeHandle hProcessingMachine,
      const CnlCommand* const pCommand ) = 0;

    // @descr   Associate an IP address and a NIC offset
    //          Preconditions:
    //            1. End machine exists
    //            2. Address not already assigned
    //            3. Address already assigned
    //          Postconditions(dependencies):
    //            a. (1,2) Association added
    //            a. (1,3) Association changed
    // @param   hProcessingMachine  IN: Handle to the end machine
    // @param   nNicId  IN: NZero-based id of Nic to assign to 
    // @param   strClasslessIpAddress  IN: Address to associate
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult AssignIpAddress(
      const CnlCompositeHandle hProcessingMachine,
      const unsigned int nNicId,
      const CString& strClasslessIpAddress ) = 0;

    // @descr   Connect the router to the specified network using the specified NIC
    //          Preconditions:
    //            1. Network exists
    //            2. NIC exists
    //          Postconditions(dependencies):
    //            a. (all) Router connected
    // @param   hNetwork  IN: Network to connect to
    // @param   nNicId  IN: Zero-based id of Nic to assign to 
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult ConnectRouter(
      const CnlCompositeHandle hNetwork,
      const unsigned int nNicId ) = 0;

    // @descr   Get a handle to the end machine that is currently the point of focus.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Handle returned
    // @return  Handle to the hot spot
    // @end
    virtual CnlCompositeHandle GetPofHandle() = 0;

    // @descr   Get the message that is associated with the current hot spot
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Message returned
    // @return  Message
    // @end
    virtual CString GetPofMessage() = 0;

    // @descr   Get the number of message that is associated with the current hot spot
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Number returned
    // @return  Sequence number
    // @end
    virtual unsigned int GetPofSequenceNum() const = 0;

    // @descr   Advance the current run by one step.
    //          Preconditions:
    //            1. A run is in progress
    //          Postconditions(dependencies):
    //            a. (all) Run is advanced
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult SingleStep() = 0;

    // @descr   Advance the current run to the end.
    //          Preconditions:
    //            1. A run is in progress
    //          Postconditions(dependencies):
    //            a. (all) Run completes
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult RunToEnd() = 0;

    virtual CString ResultToString(
      NnlSystemMgrType::EnlResult eResult ) const = 0;

    // @descr   Get a string description of the specified end machine.
    //          Preconditions:
    //            1. Machine exists
    //          Postconditions(dependencies):
    //            a. (none) Description returned; else NULL returned
    // @param   hProcessingMachine  IN: Machine to get description for
    // @return  Description
    // @end
    virtual CString GetProcessingMachineDescr(
      const CnlCompositeHandle hProcessingMachine ) const = 0;

    // @descr   Get the IP Address assigned to the specified NIC
    //          Preconditions:
    //            1. NIC exists
    //          Postconditions(dependencies):
    //            a. (none) IP Address returned; else NULL returned
    // @param   strIpAddress  OUT: Address
    // @param   hProcessingMachine  IN: Machine containing the NIC
    // @param   nNicOffset  IN: Offset of NIC to get address for
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult GetProcessingMachineIpAddress(
      CString& strIpAddress,
      const CnlCompositeHandle hProcessingMachine,
      const unsigned int nNicOffset ) const = 0;

    // @descr   Get the MAC Address assigned to the specified NIC
    //          Preconditions:
    //            1. NIC exists
    //          Postconditions(dependencies):
    //            a. (none) IP Address returned; else NULL returned
    // @param   strIpAddress  OUT: Address
    // @param   hProcessingMachine  IN: Machine containing the NIC
    // @param   nNicOffset  IN: Offset of NIC to get address for
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult GetProcessingMachineMacAddress(
      CString& strMacAddress,
      const CnlCompositeHandle hProcessingMachine,
      const unsigned int nNicOffset ) const = 0;

    // @descr   Query the specified end machine for specified console output
    //          Preconditions:
    //            1. Machine exists
    //          Postconditions(dependencies):
    //            a. (none) Results returned; else NULL returned
    // @param   nNumNics   OUT: NIC count
    // @param   hProcessingMachine  IN: Machine to get netstat for
    // @param   eEntry  IN: Type of command line entry
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult GetProcessingMachineNicCount(
      unsigned int& nNumNics,
      const CnlCompositeHandle hProcessingMachine ) const = 0;

    // @descr   Query the specified end machine for specified console output
    //          Preconditions:
    //            1. Machine exists
    //          Postconditions(dependencies):
    //            a. (none) Results returned; else NULL returned
    // @param   strConsoleOutput   OUT: Desired console output
    // @param   hProcessingMachine  IN: Machine to get netstat for
    // @param   eEntry  IN: Type of command line entry
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult GetProcessingMachineString(
      CString& strConsoleOutput,
      const CnlCompositeHandle hProcessingMachine,
      NnlSystemMgrType::EnlCommandLineEntry eEntry ) const = 0;

    // @descr   Query the specified end machine for specified console output
    //          Preconditions:
    //            1. Machine exists
    //          Postconditions(dependencies):
    //            a. (none) Results returned; else NULL returned
    // @param   strConsoleOutput   OUT: Desired console output
    // @param   hProcessingMachine  IN: Machine to get netstat for
    // @param   eEntry  IN: Type of command line entry
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult ClearProcessingMachineArpCache(
      const CnlCompositeHandle hProcessingMachine) = 0;

    // @descr   Query the specified end machine for its On/Off status
    //          Preconditions:
    //            1. Machine exists
    //          Postconditions(dependencies):
    //            a. (none) Results returned; else NULL returned
    // @param   bIsOn   OUT: True if on; False if off
    // @param   hProcessingMachine  IN: Machine to get netstat for
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult ToggleProcessingMachinePower(
      const CnlCompositeHandle hProcessingMachine ) = 0;

    // @descr   Query the specified end machine for its On/Off status
    //          Preconditions:
    //            1. Machine exists
    //          Postconditions(dependencies):
    //            a. (none) Results returned; else NULL returned
    // @param   bIsOn   OUT: True if on; False if off
    // @param   hProcessingMachine  IN: Machine to get netstat for
    // @return  Request result
    // @end
    virtual NnlSystemMgrType::EnlResult GetProcessingMachinePowerStatus(
      bool& bIsOn,
      const CnlCompositeHandle hProcessingMachine ) const = 0;

    // @descr   Identify which layers' activity you wish to view. The specified
    //          layer's and all higher layers' activity will be shown.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Filter set
    // @param   eLowestLayerToShow  IN: Filter level
    // @end
    virtual void SetLayerFilter(
      const NnlSystemMgrType::EnlLayer eLowestLayerToShow ) = 0;

    // @descr   Get the current layer filter setting
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Setting returned
    // @return  Current setting
    // @end
    virtual NnlSystemMgrType::EnlLayer GetLayerFilter() const = 0;

    // @descr   Get a string description of the specified layer
    //          Preconditions:
    //            1. Valid layer
    //          Postconditions(dependencies):
    //            a. (all) Description returned; else null string
    // @param   eLayer  IN: Layer to describe
    // @return  Description or null string
    // @end
    virtual CString GetLayerString(
      const NnlSystemMgrType::EnlLayer eLayer ) const = 0;

    // @descr   Set the flag controlling whether or not to show send-related 
    //          point-of-focus messages
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Flag set
    // @param   eEventType  IN: Event type to filter
    // @param   bShow  IN: True = show sends; False = don't
    // @end
    virtual NnlSystemMgrType::EnlResult SetShowEventType(
      NnlSystemMgrType::EnlEventType eEventType,
      const bool bShow ) = 0;
    
    // @descr   Get the current show-send setting
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Setting returned
    // @param   eEventType  IN: Event type to get show status for
    // @param   Current setting
    // @end
    virtual bool GetShowEventType(
      NnlSystemMgrType::EnlEventType eEventType ) const = 0;

    // @descr   Add a NIC to the router
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) NIC added
    // @param   hProcessingMachine  IN: Router to add to
    // @param   eNicType  IN: Type of NIC to add
    // @return  Desc
    //            (cont'd)  
    // @end
    virtual NnlSystemMgrType::EnlResult AddNicToRouter(
      const CnlCompositeHandle hProcessingMachine,
      NnlSystemMgrType::EnlRouterNicType eNicType ) = 0;

    // @descr   Identifies if a command is currently running
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Result returned
    // @return  True if a command is running; False otherwise
    // @end
    virtual bool IsCommandExecuting() const = 0;

    virtual void EnablePofProcessing( 
      const bool bEnable ) = 0;
    
    virtual void Test() = 0;

  protected:

    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlClientInterface()
    {}

    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlClientInterface()
    {}

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlClientInterface( const CnlClientInterface& );
    CnlClientInterface& operator=( const CnlClientInterface& );
    CnlClientInterface* operator&();
    const CnlClientInterface* operator&() const;
};

#endif

