/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlIPv4Address.h
//
// @short    See nlIPv4Address.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 Utilities
#include "nlCommonType.h"

//1 Local
#include "nlIPv4Address.h"
#include "nlNetworkModel.h"
#include "nlSystemMgrType.h"


///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlSystemMgrType;
using namespace NnlType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlIPv4Address::SnlImp* CnlIPv4Address::Imp()       { return m_pImp; }
inline const  CnlIPv4Address::SnlImp* CnlIPv4Address::Imp() const { return m_pImp; }

struct CnlIPv4Address::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlIPv4Address* const pOwner,
    const EnlResult eValidity, 
    const CString strClasslessAddress,
    const CString strMachineAddress,
    const CString strNetworkAddress,
    const CString strBroadcastAddress,
    const CString strGatewayAddress,
    const int nLength );

  // Copy constructor
  SnlImp( 
    CnlIPv4Address* const pOwner,
    const SnlImp& rhs );

  // Equivalence operator
  bool operator==(
    const SnlImp& rhs ) const;
  
  // Less than operator
  bool operator<(
    const SnlImp& rhs ) const;
  
  // Destructor
   ~SnlImp();

  // @descr   Convert and evaluate and the owner's ctor parms 
  //          Preconditions:
  //            1. Valid formats
  //          Postconditions(dependencies):
  //            a. (none) Parms evaluated
  //            b. (all) Parms converted
  // @param   eValidity  OUT: Validity of the parms
  // @param   strAddress  OUT: The machine's address in dot notation
  // @param   nLength  OUT: Network address length
  // @param   strClassfullAddress  IN: Address to convert
  // @param   eAddressType  IN: Type of address
  // @end
  static void ConvertAndValidateParms(
    EnlResult& eValidity,
    CString& strMachineAddress,
    CString& strNetworkAddress,
    CString& strBroadcastAddress,
    CString& strGatewayAddress,
    int& nLength,
    const CString& strClasslessAddress );
    
  // @descr   Parse a classless address into its component strings
  //          Preconditions:
  //            1. Valid format
  //          Postconditions(dependencies):
  //            a. (all) Parsed components output
  // @param   strParsedAddress  OUT: Parsed address
  // @param   strParsedLength  OUT: Parsed length
  // @param   strAddressToParse  IN: String to parse
  // @return  Request result
  // @end
  static EnlResult ParseClasslessAddress(
    CString& strParsedAddress,
    CString& strParsedLength,
    const CString& strAddressToParse );

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlIPv4Address* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // Address validity
  const EnlResult m_eValidity;
  
  // Classless CString-formatted address
  const CString m_strClasslessAddress;

  // Classfull machine address
  const CString m_strMachineAddress;

  // Classfull network address
  const CString m_strNetworkAddress;

  // Classfull broadcast address
  const CString m_strBroadcastAddress;

  // Classfull gateway address
  const CString m_strGatewayAddress;

  // Length of the address portion of the bits
  const int m_nLength;

private:

  //1 Unimplemented
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlIPv4Address::SnlImp::SnlImp(
  CnlIPv4Address* const pOwner,
  const EnlResult eValidity, 
  const CString strClasslessAddress,
  const CString strMachineAddress,
  const CString strNetworkAddress,
  const CString strBroadcastAddress,
  const CString strGatewayAddress,
  const int nLength )
 :
  m_pOwner( pOwner ),
  m_eValidity( eValidity ),
  m_strClasslessAddress( strClasslessAddress ),
  m_strMachineAddress( strMachineAddress ),
  m_strNetworkAddress( strNetworkAddress ),
  m_strBroadcastAddress( strBroadcastAddress ),
  m_strGatewayAddress( strGatewayAddress ),
  m_nLength( nLength )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlIPv4Address::SnlImp::SnlImp( 
  CnlIPv4Address* const pOwner,
  const SnlImp& rhs )
 :
  m_pOwner( pOwner ),
  m_eValidity( rhs.m_eValidity ),
  m_strClasslessAddress( rhs.m_strClasslessAddress ),
  m_strMachineAddress( rhs.m_strMachineAddress ),
  m_strNetworkAddress( rhs.m_strNetworkAddress ),
  m_strBroadcastAddress( rhs.m_strBroadcastAddress ),
  m_strGatewayAddress( rhs.m_strGatewayAddress ),
  m_nLength( rhs.m_nLength )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlIPv4Address::SnlImp::~SnlImp()
{
}
///////////////////////////////////////////////////////////////////////////////
bool CnlIPv4Address::SnlImp::operator==(
  const SnlImp& rhs ) const
{
  const bool bResult =
     ( ( m_eValidity == rhs.m_eValidity )
    && ( m_strClasslessAddress == rhs.m_strClasslessAddress )
    && ( m_strMachineAddress == rhs.m_strMachineAddress )
    && ( m_strNetworkAddress == rhs.m_strNetworkAddress )
    && ( m_strBroadcastAddress == rhs.m_strBroadcastAddress )
    && ( m_strGatewayAddress == rhs.m_strGatewayAddress )
    && ( m_nLength == rhs.m_nLength ) );

  return bResult;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIPv4Address::SnlImp::operator<(
  const SnlImp& rhs ) const
{
  return ( m_strClasslessAddress < rhs.m_strClasslessAddress );
}

///////////////////////////////////////////////////////////////////////////////
void CnlIPv4Address::SnlImp::ConvertAndValidateParms(
  EnlResult& eValidity,
  CString& strMachineAddressOut,
  CString& strNetworkAddressOut,
  CString& strBroadcastAddressOut,
  CString& strGatewayAddressOut,
  int& nLengthOut,
  const CString& strClasslessAddress )
{
  //2 Parse the address and length
  CString strParsedAddress;
  CString strParsedLength;
  eValidity = SnlImp::ParseClasslessAddress(
    strParsedAddress,
    strParsedLength,
    strClasslessAddress );

  if ( eValidity != RESULT_OK )
  {
    //3 Could not parse the classless address
    return;
  }

  //2 Convert/validate length
  const int nLength = atoi( strParsedLength );
  
  if ( ( nLength < 0 ) || ( nLength > nNumBitsInIPv4Address ) )
  {
    eValidity = RESULT_ERROR_INVALID_ADDRESS_LENGTH;
    return;
  }

  if ( ! CnlSystemMgr::Instance().GetInternalInterface()->IsValidDotNotation(
    strParsedAddress, IP_VERSION_4 ) )
  {
    eValidity = RESULT_ERROR_INVALID_ADDRESS;
    return;
  }

  const CnlNetworkModel* pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  
  //1 Generate the network address
  unsigned long nNetworkAddress;
  pNetworkModel->DotToNumeric(
    nNetworkAddress,
    strParsedAddress,
    IP_VERSION_4 );

  //2 Zero out bits past the address length
  const unsigned long nMask = ( ( 1 << nLength ) - 1 );
  const unsigned long nAlignedMask = nMask << ( nNumBitsInIPv4Address - nLength );
  nNetworkAddress &= nAlignedMask;
  
  CString strNetworkAddress;
  pNetworkModel->NumericToDot(
    strNetworkAddress,
    nNetworkAddress,
    IP_VERSION_4 );

  //1 Generate the broadcast address
  unsigned long nBroadcastBits = ( 1 << (nNumBitsInIPv4Address - nLength ) ) - 1;
  unsigned long nBroadcastAddress = nNetworkAddress |= nBroadcastBits;

  CString strBroadcastAddress;
  pNetworkModel->NumericToDot(
    strBroadcastAddress,
    nBroadcastAddress,
    IP_VERSION_4 );
  
  //1 Generate the gateway address
  //3 By policy, the gateway address is one less than the broadcast address
  unsigned long nGatewayAddress = nBroadcastAddress - 1;
  
  CString strGatewayAddress;
  pNetworkModel->NumericToDot(
    strGatewayAddress,
    nGatewayAddress,
    IP_VERSION_4 );
  
  //2 Set outputs
  strMachineAddressOut = strParsedAddress;
  strNetworkAddressOut = strNetworkAddress;
  strBroadcastAddressOut = strBroadcastAddress;
  strGatewayAddressOut = strGatewayAddress;
  nLengthOut = nLength;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlIPv4Address::SnlImp::ParseClasslessAddress(
  CString& strParsedAddress,
  CString& strParsedLength,
  const CString& strAddressToParse )
{
  CString strWhole = strAddressToParse;
  LPTSTR pstrWhole = strWhole.GetBuffer( 0 );
  TCHAR pcWholeSeparator[] = _T( "/" );

  //1 Parse the address and length strings
  strParsedAddress = _tcstok( pstrWhole, pcWholeSeparator );

  if ( strParsedAddress == NnlType::STR_NULL )
  {
    //3 Null string!
    return RESULT_ERROR_INVALID_ADDRESS;
  }

  strParsedLength =_tcstok( NULL, pcWholeSeparator );

  if ( strParsedLength == NnlType::STR_NULL )
  {
    //3 Missing length term
    return RESULT_ERROR_INVALID_ADDRESS_LENGTH;
  }
   
  strWhole.ReleaseBuffer( -1 );  
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlIPv4Address::CnlIPv4Address()
{
  m_pImp = new SnlImp(
    this, 
    RESULT_ERROR_LEVEL, 
    STR_NULL, 
    STR_NULL, 
    STR_NULL, 
    STR_NULL, 
    STR_NULL, 
    -1 );
}

CnlIPv4Address::CnlIPv4Address(
  const CString& strClasslessAddress )
{
  EnlResult eValidity;
  CString strMachineAddress;
  CString strNetworkAddress;
  CString strBroadcastAddress;
  CString strGatewayAddress;
  int nLength = -1;

  SnlImp::ConvertAndValidateParms(
    eValidity,
    strMachineAddress,
    strNetworkAddress,
    strBroadcastAddress,
    strGatewayAddress,
    nLength,
    strClasslessAddress );  

  if ( eValidity == RESULT_OK )
  {
    m_pImp = new SnlImp(
      this, 
      eValidity, 
      strClasslessAddress, 
      strMachineAddress, 
      strNetworkAddress, 
      strBroadcastAddress, 
      strGatewayAddress, 
      nLength );
  }
  else
  {
    m_pImp = new SnlImp(
      this, 
      eValidity, 
      STR_NULL, 
      STR_NULL, 
      STR_NULL, 
      STR_NULL, 
      STR_NULL, 
      -1 );
  }
}

///////////////////////////////////////////////////////////////////////////////
CnlIPv4Address::CnlIPv4Address(
  const CnlIPv4Address& rhs )
{
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
}

///////////////////////////////////////////////////////////////////////////////
CnlIPv4Address::~CnlIPv4Address()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
CnlIPv4Address& CnlIPv4Address::operator=(
  const CnlIPv4Address& rhs )
{
  delete m_pImp;
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
  return *this;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIPv4Address::operator==(
  const CnlIPv4Address& rhs ) const
{
  bool bResult = m_pImp->operator==( * ( rhs.m_pImp ) );
  return bResult;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIPv4Address::operator!=(
  const CnlIPv4Address& rhs ) const
{
  const bool bResult = !( *m_pImp == ( * ( rhs.m_pImp ) ) );
  return bResult;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIPv4Address::operator<(
  const CnlIPv4Address& rhs ) const
{
  const bool bResult = ( *m_pImp < ( * ( rhs.m_pImp ) ) );
  return bResult;
}


///////////////////////////////////////////////////////////////////////////////
NnlSystemMgrType::EnlResult CnlIPv4Address::Validity()
{
  return Imp()->m_eValidity;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIPv4Address::Classless() const
{
  return Imp()->m_strClasslessAddress;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIPv4Address::Network() const
{
  return Imp()->m_strNetworkAddress;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIPv4Address::Machine() const
{
  return Imp()->m_strMachineAddress;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIPv4Address::Broadcast() const
{
  return Imp()->m_strBroadcastAddress;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIPv4Address::Gateway() const
{
  return Imp()->m_strGatewayAddress;
}

///////////////////////////////////////////////////////////////////////////////
int CnlIPv4Address::Length() const
{
  return Imp()->m_nLength;
}


