/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlIpModule.h
//
// @short    See nlIpModule.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>
#include <nlVariableParmContainer.h>

//1 Local
#include "nlIpModule.h"
#include "nlPacket.h"
#include "nlLinkLayerInterface.h"
#include "nlRoutingTable.h"
#include "nlRoutingTableRow.h"
#include "nlIPv4Address.h"
#include "nlSystemMgrType.h"
#include "nlOs.h"
#include "nlHwSwInterface.h"
#include "nlSegment.h"
#include "nlCompositeHandle.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::map< CString, unsigned int >::const_iterator IpNicMapConstIter;
  typedef std::set< CString >::const_iterator IPv4SetConstIter;
  typedef std::map< CString, CString >::const_iterator ArpCacheIter;

  const CString strArpCacheTitle = _T("arp -a" );
  const CString strArpCacheEmpty = _T("No ARP Entries Found" );
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlIpModule::SnlImp* CnlIpModule::Imp()       { return m_pImp; }
inline const  CnlIpModule::SnlImp* CnlIpModule::Imp() const { return m_pImp; }

struct CnlIpModule::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlIpModule* const pOwner,
    CnlOs* const pOs );

  // Destructor
   ~SnlImp();

  void SendRarp(
    const CString& strDestinationIp );

  bool IsForMe(
    const CString& strDestinationIp );
  
  // Identify the MAC address to use to send to the specified IP
  CString GetMacFromIp(
    const CString& strIpAddress );
  
  // Lookup an IP address in the cache and get the corresponding MAC
  CString GetMacFromCache(
    const CString strIpAdress ) const;

  // Add an entry to the ARP cache
  void AddToArpCache(
    const CString strIpAdress,
    const CString strMacAdress );

  void DumpCache() const;

  void QueryForRouterIp();


  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlIpModule* const m_pOwner;

  // The Operating system core
  CnlOs* const m_pOs;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The Arp Module
//  CnlArpModule* m_pArpModule;
  
  // The Link layer moules I work with
  CnlLinkLayerInterface* m_pEthernetModule;

  // The IP routing table
  CnlRoutingTable* const m_pRoutingTable;

  // The ARP cache (IP Address to MAC address mapping)
  std::map< CString, CString > m_ArpCache;

  // My Router's IP address (if any)
  CString m_strRouterIpAddress;

  // What is the progress of the current command?
  EnlProgress m_eProgress;
  
private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlIpModule::SnlImp::SnlImp(
  CnlIpModule* const pOwner,
  CnlOs* const pOs )
 :
  m_pOwner( pOwner ),
  m_pOs( pOs ),
  m_pEthernetModule( NULL ),
  m_pRoutingTable( new CnlRoutingTable() ),
  m_eProgress( PROGRESS_NOT_STARTED )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlIpModule::SnlImp::~SnlImp()
{
}


///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::SnlImp::SendRarp(
  const CString& strDestinationIp )
{
  //1 Error checking
  if ( m_pOwner->IsMyIpAddress( strDestinationIp ) )
  {
    //2 Don't Rarp myself
    return;
  }

  const CnlPacket packet = m_pOwner->MakePacket(
    strDestinationIp, 
    TRANSPORT_PROTOCOL_NONE, 
    CnlSegment() );
  
  const bool bResult = m_pOwner->SendPacket( 
    packet, 
    NETWORK_PROTOCOL_RARP,
    false );
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIpModule::SnlImp::IsForMe(
  const CString& strDestinationIp )
{
  const bool bIsMine = m_pOwner->IsMyIpAddress( strDestinationIp );
  return bIsMine;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIpModule::SnlImp::GetMacFromIp(
  const CString& strIpAddress )
{
  //1 Check the ARP cache
  CString& strMacAddress = GetMacFromCache( strIpAddress );

  if ( strMacAddress != STR_NULL )
  {
    //2 Found it
    return strMacAddress;
  }
 
  //2 Not in the cache, send ARP
  m_pOwner->SendArp( strIpAddress );

  //2 Check again
  strMacAddress = GetMacFromCache( strIpAddress );

  if ( strMacAddress != STR_NULL )
  {
    //2 Found it
    return strMacAddress;
  }
  else
  {
    //2 Not in this network
    return STR_NULL;
  }
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIpModule::SnlImp::GetMacFromCache(
  const CString strIpAddress ) const
{
  TRACE1( _T( "\nLooking up IP: %s in cache: " ), strIpAddress );
  DumpCache();

  ArpCacheIter iter = m_ArpCache.find( strIpAddress );

  if ( iter != m_ArpCache.end() )
  {
    //2 Found it in the cache
    return ( *iter ).second;
  }
  else
  {
    //2 Not in the cache
    return STR_NULL;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::SnlImp::AddToArpCache(
  const CString strIpAdress,
  const CString strMacAdress )
{
  m_ArpCache[ strIpAdress ] = strMacAdress;
  DumpCache();
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::SnlImp::DumpCache() const
{
  TRACE1( 
    _T( "\nEnd Machine %d ARP cache:" ),
    m_pOs->GetHardwareInterface()->GetProcessingMachineHandle().ID() );

  for ( ArpCacheIter iter = m_ArpCache.begin(); iter != m_ArpCache.end(); ++iter )
  {
    TRACE2(
      _T( "\n%s %s" ), ( *iter ).first, ( *iter ).second );
  }
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlIpModule::CnlIpModule(
  CnlOs* const pOs )
{
  m_pImp = new SnlImp( this, pOs );
}

///////////////////////////////////////////////////////////////////////////////
CnlIpModule::~CnlIpModule()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::Initialize(
//  CnlArpModule* const pArpModule,
  CnlLinkLayerInterface* const pEthernetModule )
{
//  if ( Imp()->m_pArpModule == NULL )
//  {
//    Imp()->m_pArpModule = pArpModule;
//  }
  if ( Imp()->m_pEthernetModule == NULL )
  {
    Imp()->m_pEthernetModule = pEthernetModule;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlIpModule::AssignIpAddress(
  const unsigned int nNicId,
  const CString& strClasslessIpAddress )
{
  const EnlResult eResult = 
    Imp()->m_pRoutingTable->AssignIpAddress( nNicId, strClasslessIpAddress );

  if ( eResult != RESULT_OK )
  {
    return eResult;
  }

  //1 Whenever the interfaces change, rebuild the routing table
  BuildRoutingTable();

  return RESULT_OK;
}

#if 0
///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::Broadcast(
  const NnlSystemMgrType::EnlTransportProtocol eSegmentType,
  const CnlSegment& segment)
{
  //1 Make an IP packet this segment
  CnlPacket packet( eSegmentType, segment );

  //1 Send it down  
  Imp()->m_pEthernetModule->BroadcastFrame( NETWORK_PROTOCOL_IP, packet );
}
///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::SendToIpAddress(
  const NnlSystemMgrType::EnlTransportProtocol eSegmentType,
  const CnlSegment& segment,
  const CString strDestinationAddress )
{
  //1 Make an IP packet this segment
  CnlPacket packet( eSegmentType, segment );

  //1 Send it down  
//!!!jc  LEFT OFF HERE

//  const CString& strInterfaceIp = GetInterfaceFromRoutingTable( strDestinationAddress );
//  const CString& strMacAddress = Imp()->m_pos GetNicFromInterface( strInterfaceIp );

//  Imp()->m_pEthernetModule->SendToMacAddress( NETWORK_PROTOCOL_IP, packet strMacAddress );
}
#endif  // #if 0

///////////////////////////////////////////////////////////////////////////////
bool CnlIpModule::IsMyIpAddress(
  const CString& strIpAddress ) const
{
  TRACE1( _T( "\nIs IPv4 %s mine?" ), strIpAddress );

  return ( Imp()->m_pRoutingTable->IsMyIpAddress( strIpAddress ) );
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::SendArp(
  const CString& strDestinationIp )
{
  Imp()->m_eProgress = PROGRESS_STARTED;

  //1 Error checking
  if ( IsMyIpAddress( strDestinationIp ) )
  {
    //2 Don't Arp myself
    return;
  }

  const CnlPacket packet = MakePacket(
    strDestinationIp, 
    TRANSPORT_PROTOCOL_NONE, 
    CnlSegment() );
  
  const bool bResult = SendPacket( 
    packet, 
    NETWORK_PROTOCOL_ARP,
    false );
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::ReceiveArp(
  CnlPacket payload,
  const CString& strSourceMac )
{
  //2 Reject if not for me
  if ( ! Imp()->IsForMe( payload.GetDestinationIpAddress() ) )
  {
    return;
  }
  
  //2 Add sender to ARP cache
  Imp()->AddToArpCache( payload.GetSourceIpAddress(), strSourceMac );
  Imp()->SendRarp( payload.GetSourceIpAddress() );
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::ReceiveRarp(
  CnlPacket payload,
  const CString& strSourceMac )
{
  //2 Reject if not for me
  if ( ! Imp()->IsForMe( payload.GetDestinationIpAddress() ) )
  {
    return;
  }
  
  //2 Add sender to ARP cache
  Imp()->AddToArpCache( payload.GetSourceIpAddress(), strSourceMac );
  Imp()->m_eProgress = PROGRESS_COMPLETED;
}

 ///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::SendPing(
  const CString& strDestinationIp )
{
  Imp()->m_eProgress = PROGRESS_STARTED;

  //1 Error checking
  if ( Imp()->m_pRoutingTable->IsMyIpAddress( strDestinationIp ) )
  {
    //2 Don't Ping myself
    return;
  }

  const CnlPacket packet = MakePacket( 
    strDestinationIp, 
    TRANSPORT_PROTOCOL_PING, 
    CnlSegment() );

  const bool bResult = SendPacket( 
    packet,
    NETWORK_PROTOCOL_IP,
    true );
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::VerifyArpResult(
  const CString& strDestinationIp )
{
  //2 Let the user know how it turned out
  //3 Pass or fail, they haven't seen anything yet
  CString strMessage;
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  
  if( Imp()->m_eProgress == PROGRESS_COMPLETED )
  {
   strMessage.Format( 
      _T( "Reply received from %s; ARP cache entry made." ),
      strDestinationIp );
  }
  else
  {
   strMessage.Format( 
      _T( "No reply received from %s." ),
      strDestinationIp );
  }

  pNetworkModel->SuspendAtPof( 
    Imp()->m_pOs->GetHardwareInterface()->GetProcessingMachineHandle(), 
    LAYER_UPPER,
    EVENT_TYPE_INFO,
    strMessage );

  Imp()->m_eProgress = PROGRESS_COMPLETED;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::VerifyPingResult(
  const CString& strDestinationIp )
{
  //2 Let the user know if it timed out
  //3 If it passed, they've already seen the result
  if( Imp()->m_eProgress == PROGRESS_COMPLETED )
  {
    return;
  }

  CString strMessage;
  strMessage.Format(
    _T( "ping %s: No reply received." ), strDestinationIp );
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();

  pNetworkModel->SuspendAtPof( 
    Imp()->m_pOs->GetHardwareInterface()->GetProcessingMachineHandle(), 
    LAYER_UPPER,
    EVENT_TYPE_INFO,
    strMessage );

  Imp()->m_eProgress = PROGRESS_COMPLETED;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIpModule::GetNetstatString() const
{
  return Imp()->m_pRoutingTable->AsString();
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::SendRping(
  const CString& strAddressToReplyTo )
{
  //1 Error checking
  if ( Imp()->m_pRoutingTable->IsMyIpAddress( strAddressToReplyTo ) )
  {
    //2 Don't Ping myself
    return;
  }


  const CnlPacket packet = MakePacket( 
    strAddressToReplyTo, 
    TRANSPORT_PROTOCOL_RPING, 
    CnlSegment() );

  const bool bResult = SendPacket( 
    packet,
    NETWORK_PROTOCOL_IP,
    true );
}

///////////////////////////////////////////////////////////////////////////////
CnlOs* CnlIpModule::GetOs() const
{
  return Imp()->m_pOs;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIpModule::ReceivePacket(
  const CnlPacket& packet )
{
  if ( ! Imp()->IsForMe( packet.GetDestinationIpAddress() ) )
  {
    return false;
  }
  
  switch( packet.GetTransportProtocol() )
  {
    case TRANSPORT_PROTOCOL_PING :
    {
      //2 I've been pinged; reply
      SendRping( packet.GetSourceIpAddress() );
      return true;
    }
    case TRANSPORT_PROTOCOL_RPING :
    {
      //2 Received a reply to my ping
      CString strConsoleMessage;
      strConsoleMessage.Format( 
        _T( "Reply from %s:  bytes=32  time<10ms  TTL=255" ),
        packet.GetSourceIpAddress() );
      GetOs()->SetConsoleMessage( strConsoleMessage );
      Imp()->m_eProgress = PROGRESS_COMPLETED;
      return true;
    }
    case TRANSPORT_PROTOCOL_ROUTER_RESPONSE :
    {
      //2 Received a reply to my router query
      Imp()->m_strRouterIpAddress = packet.GetSourceIpAddress();
      return true;
    }
    default : 
    {
    }
  }

  return false;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::BuildRoutingTable()
{
  Imp()->m_strRouterIpAddress = STR_NULL;

  //2 Build an initial table
  Imp()->m_pRoutingTable->ClearTable();
  this->FillRoutingTable();

  //2 See if there is a router for this network address
  QueryForRouterIp();

  //2 Incorporate results into final table
  Imp()->m_pRoutingTable->ClearTable();
  this->FillRoutingTable();
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::AddRowToRoutingTable(
  const CString& strDestination,
  const CString& strNetMask,
  const CString& strGateway,
  const CString& strInterface )
{
  Imp()->m_pRoutingTable->AddRow(
    strDestination, 
    strNetMask, 
    strGateway, 
    strInterface );
}
 
///////////////////////////////////////////////////////////////////////////////
EnlResult CnlIpModule::GetIpAddress(
  CnlIPv4Address& address,
  const unsigned int nNicOffset ) const
{
  return Imp()->m_pRoutingTable->GetIpAddress( address, nNicOffset );
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIpModule::Lookup( 
  CString& strGateway,
  CString& strInterface,
  const CString& strIpToLookup ) const
{
  return Imp()->m_pRoutingTable->Lookup( strGateway, strInterface, strIpToLookup );
}

///////////////////////////////////////////////////////////////////////////////
unsigned int CnlIpModule::GetNicOffsetFromInterface(
  const CString& strInterface ) const
{
  return Imp()->m_pRoutingTable->GetNicOffsetFromInterface( strInterface );
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIpModule::GetCacheString()
{
  CString strArpCache = strArpCacheTitle;

  for ( ArpCacheIter iter = Imp()->m_ArpCache.begin();
        iter != Imp()->m_ArpCache.end();
        ++iter )
  {
    CString strCacheLine;
      strCacheLine.Format( 
        _T( "\n%s\t%s " ), 
        ( *iter ).first, 
        ( *iter ).second );

    strArpCache += strCacheLine;
  }

  if ( strArpCache == strArpCacheTitle )
  {
    strArpCache = strArpCache + STR_NEWLINE + strArpCacheEmpty;
  }
  
  return strArpCache;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModule::ClearArpCache()
{
  Imp()->m_ArpCache.clear();
}

///////////////////////////////////////////////////////////////////////////////
CnlPacket CnlIpModule::MakePacket(
  const CString& strDestinationIp, 
  EnlTransportProtocol eTransportProtocol, 
  const CnlSegment& segment )
{
  //2 Determine the Source IP to use
  CString strGateway;
  CString strInterface;
  const bool bFound = 
    Imp()->m_pRoutingTable->Lookup( 
      strGateway,
      strInterface,
      strDestinationIp );

  if ( ! bFound )
  {
    //3 Asked for an address I can't get to
    return CnlPacket();  // An invalid packet
  }

  return CnlPacket( strInterface, strDestinationIp, eTransportProtocol, segment );
}


///////////////////////////////////////////////////////////////////////////////
bool CnlIpModule::SendPacket(
  const CnlPacket& packet,
  const EnlNetworkProtocol eNetworkProtocol,
  const bool bShowMesage )
{
  if ( ! packet.IsValid() )
  {
    //3 Can't send this packet
    return false;
  }
  
  CString strGateway;
  CString strInterface;
  const bool bFound = Lookup( 
      strGateway,
      strInterface,
      packet.GetDestinationIpAddress() );

  if ( ! bFound )
  {
    //3 Asked for an address I can't get to
    return false;  // An invalid packet
  }

  if ( strInterface == strInterfaceLoopBack )
  {
    //2 Loopback
    return false;
  }

  const CString strDestinationIp = packet.GetDestinationIpAddress();

  //2 Announce it
  if ( bShowMesage )
  {
    CnlNetworkModel* const pNetworkModel = 
      CnlSystemMgr::Instance().GetInternalInterface();
    const CString strProtocol = 
      pNetworkModel->TransportProtocolToString( packet.GetTransportProtocol() );
    CString strMessage;
    strMessage.Format( 
      _T( "Send IP packet: " )
      _T( "Transport Protocol: %s, " )
      _T( "SourceIp: %s, " )
      _T( "DestinationIp: %s" ),
      strProtocol,
      packet.GetSourceIpAddress(),
      strDestinationIp );

    pNetworkModel->SuspendAtPof( 
      Imp()->m_pOs->GetHardwareInterface()->GetProcessingMachineHandle(), 
      LAYER_NETWORK,
      EVENT_TYPE_SEND,
      strMessage );
  }
  
  //1 Send it out a NIC
  const CString& strIpBroadcast = 
    Imp()->m_pRoutingTable->GetBroadcastAddressFromInterface( strInterface );
  const bIpBroadcast = 
    (  ( packet.GetDestinationIpAddress() == strIpBroadcast )
    || ( eNetworkProtocol == NETWORK_PROTOCOL_ARP ) );

  const unsigned int nNicOffset = GetNicOffsetFromInterface( strInterface );
  CString strDestinationMac;
  
  if ( bIpBroadcast )
  {
    Imp()->m_pEthernetModule->BroadcastFrame( 
      nNicOffset,
      eNetworkProtocol, 
      packet );
  }
  else
  {
    if ( strInterface == strGateway )
    {
      //3 Direct to destination
      strDestinationMac = Imp()->GetMacFromIp( strDestinationIp );
    }
    else
    {
      //3 Through router
      strDestinationMac = Imp()->GetMacFromIp( GetRouterIp() );
    }

    if ( strDestinationMac == STR_NULL )
    {
      //2 Don't know where to send to
      return false;
    }

    Imp()->m_pEthernetModule->SendFrameToMachine(
      nNicOffset,
      strDestinationMac,
      eNetworkProtocol, 
      packet );
  }

  return true;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlIpModule::GetRouterIp() const
{
  return Imp()->m_strRouterIpAddress;
}

