//////////////////////////////////////////////////////////////////////////////
//
// @descr    Base class for all IP modules
//
// @short    Base class for all IP modules
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_IP_MODULE_H__
#define __NL_IP_MODULE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlNetworkLayerInterface.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlPacket;
class CnlIPv4Address;
class CnlArpModule;
class CnlOs;
class CnlVariableParmContainer;
class CnlTransportLayerInterface;
class CnlLinkLayerInterface;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlIpModule : public CnlNetworkLayerInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @param  pOs  IN: The operating system core
    // @end
    CnlIpModule(
      CnlOs* const pOs );
    
    // @descr  Post-constructor initialization
    // @end
    void Initialize(
//      CnlArpModule* const pArpModule,
      CnlLinkLayerInterface* const pEthernetModule );

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    // @descr   Assign an IP address to the specified NIC offset position
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (all) Assignment made
    // @param   nNicId  IN: NIC offset
    // @param   address  IN: Address to assign
    // @end
    NnlSystemMgrType::EnlResult AssignIpAddress(
      const unsigned int nNicId,
      const CString& address );

    // @descr   Make an entry in the routing table
    //          Preconditions:
    //            1. Ip address not already assigned
    //          Postconditions(dependencies):
    //            a. (none) Entry made
    // @param   nX  OUT: Desc
    //            (cont'd)
    // @param   pY  IN: Desc
    //            (cont'd)  
    // @return  Desc
    //            (cont'd)  
    // @end
    void AddToRoutingTable(
      const CnlIPv4Address& address );

    // @descr   Get a string representing the console output from a netstat -rn command.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (nonel) String returned
    // @return  Netstat -rn output
    // @end
    CString GetNetstatString() const;

    // @descr   Simulate Arp execution
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Arp executed for the specified IP address
    // @param   strDestinationIp  IN: The IP address to ARP
    // @end
    void SendArp(
      const CString& strDestinationIp );

    // @descr   Simulate Ping execution
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Ping executed for the specified IP address
    // @param   strDestinationIp  IN: The IP address to ping
    // @end
    void SendPing(
      const CString& strDestinationIp );

    NnlSystemMgrType::EnlResult GetIpAddress(
      CnlIPv4Address& address,
      const unsigned int nNicOffset ) const;

    void VerifyArpResult( 
      const CString& strDestinationIp );
    void VerifyPingResult( 
      const CString& strDestinationIp );
    
    //2 From CnlNetworkLayerInterface
//    void Broadcast(
//      const NnlSystemMgrType::EnlSegmentProtocol,
//      const CnlSegment& );
//    void SendToIpAddress(
//      const NnlSystemMgrType::EnlSegmentProtocol ,
//      const CnlSegment& ,
//      const CString );
    bool IsMyIpAddress(
      const CString& ) const;
    bool Lookup( 
      CString&,
      CString&,
      const CString& ) const;
    unsigned int GetNicOffsetFromInterface( 
      const CString& ) const;
    void ReceiveArp(
      CnlPacket,
      const CString& );
    void ReceiveRarp(
      CnlPacket,
      const CString& );



    //2 Forwarded from OS
    CString GetCacheString();
    void ClearArpCache();


    //2 From Cnl
    
    //1 Virtual
    // @descr   Destructor
    // @end
    virtual ~CnlIpModule();

    //2 From CnlNetworkLayerInterface
    virtual bool ReceivePacket(
      const CnlPacket& packet );

    //1 Abstract

    // Generate the IP Routing Table
    void BuildRoutingTable();


  protected:

    //1 Final

    CnlOs* GetOs() const;
    
    void SendRping(
      const CString& strAddressToReplyTo );

    // Append the specfied row to the routing table
    void AddRowToRoutingTable(
      const CString& strDestination,
      const CString& strNetMask,
      const CString& strGateway,
      const CString& strInterface );

    CnlPacket MakePacket(
      const CString& strDestinationIp, 
      NnlSystemMgrType::EnlTransportProtocol eTransportProtocol, 
      const CnlSegment& segment );


    // Send the specified packet
    bool SendPacket(
      const CnlPacket& packet,
      const NnlSystemMgrType::EnlNetworkProtocol eNetworkProtocol,
      const bool bSHowMessage );

    // Identify the IP Address of the Gateway Router, if one exists
    CString GetRouterIp() const;


    //1 Abstract
    // Get the Gateway IP address from the derived class
//    virtual CString GetRouterIp() const = 0;
  
    // Get the Gateway interface from the derived class
//    virtual CString GetGatewayInterface() const = 0;

    // Have derived class figure out who its gateway is
    virtual void QueryForRouterIp() = 0;
    virtual void FillRoutingTable() = 0;

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlIpModule( const CnlIpModule& );
    CnlIpModule& operator=( const CnlIpModule& );
    CnlIpModule* operator&();
    const CnlIpModule* operator&() const;
};

#endif

