/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlIpModuleEndMachine.h
//
// @short    See nlIpModuleEndMachine.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>
#include <nlCompositeHandle.h>
//1 Local
#include "nlIpModuleEndMachine.h"
#include "nlSystemMgrType.h"
#include "nlPacket.h"
#include "nlIPv4Address.h"
#include "nlOs.h"
#include "nlTransportLayerInterface.h"
#include "nlSegment.h"
#include "nlHwSwInterface.h"
#include "nlNetworkModel.h"


///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::map< CString, unsigned int >::const_iterator IpNicMapConstIter;
  typedef std::set< CString >::const_iterator IPv4SetConstIter;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlIpModuleEndMachine::SnlImp* CnlIpModuleEndMachine::Imp()       { return m_pImp; }
inline const  CnlIpModuleEndMachine::SnlImp* CnlIpModuleEndMachine::Imp() const { return m_pImp; }

struct CnlIpModuleEndMachine::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlIpModuleEndMachine* const pOwner );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlIpModuleEndMachine* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The Transport layer modules I work with
  CnlTransportLayerInterface* m_pUdpModule;
  CnlTransportLayerInterface* m_pTcpModule;

  // The IP address assigned to my NIC
  CnlIPv4Address m_IpAddress;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlIpModuleEndMachine::SnlImp::SnlImp(
  CnlIpModuleEndMachine* const pOwner )
 :
  m_pOwner( pOwner ),
  m_pUdpModule( NULL ),
  m_pTcpModule( NULL )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlIpModuleEndMachine::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlIpModuleEndMachine::CnlIpModuleEndMachine(
  CnlOs* const pOs )
 :
  CnlIpModule( pOs )
{
  m_pImp = new SnlImp( this );
}

///////////////////////////////////////////////////////////////////////////////
CnlIpModuleEndMachine::~CnlIpModuleEndMachine()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModuleEndMachine::Initialize(
  CnlTransportLayerInterface* const pUdpModule,
  CnlTransportLayerInterface* const pTcpModule,
//  CnlArpModule* const pArpModule,
  CnlLinkLayerInterface* const pEthernetModule )
{
  CnlIpModule::Initialize( pEthernetModule );

  if ( Imp()->m_pUdpModule == NULL )
  {
    Imp()->m_pUdpModule = pUdpModule;
  }
  if ( Imp()->m_pTcpModule == NULL )
  {
    Imp()->m_pTcpModule = pTcpModule;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModuleEndMachine::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIpModuleEndMachine::ReceivePacket(
  const CnlPacket& packet )
{
  if ( ! IsMyIpAddress( packet.GetDestinationIpAddress() ) )
  {
    //3 Not one of my IP addresses
    return false;
  }

  //2 Announce it
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  const CString strProtocol = 
    pNetworkModel->TransportProtocolToString( packet.GetTransportProtocol() );
  CString strMessage;
  strMessage.Format( 
    _T( "Receive IP packet:" )
    _T( "Transport Protocol: %s, " )
    _T( "SourceIp: %s, " )
    _T( "DestinationIp: %s" ),
    strProtocol,
    packet.GetSourceIpAddress(),
    packet.GetDestinationIpAddress() );

  pNetworkModel->SuspendAtPof( 
    GetOs()->GetHardwareInterface()->GetProcessingMachineHandle(), 
    LAYER_NETWORK,
    EVENT_TYPE_RECEIVE_ACCEPT,
    strMessage );

if ( packet.GetDestinationIpAddress() == _T( "192.168.1.1" ) )
{
  int nBreakhere = 1;
}
  const bool bHandled = CnlIpModule::ReceivePacket( packet );

  if ( bHandled )
  {
    return true;
  }

  switch( packet.GetTransportProtocol() )
  {
    case TRANSPORT_PROTOCOL_UDP :
    {
      Imp()->m_pUdpModule->ReceiveSegment( packet.GetPayload() );
      return true;
    }
    case TRANSPORT_PROTOCOL_TCP :
    {
      Imp()->m_pTcpModule->ReceiveSegment( packet.GetPayload() );
      return true;
    }
    case TRANSPORT_PROTOCOL_ROUTER_QUERY :
    {
      //3 I'm not a router; don't respond
      return false;
    }
    default : 
    {
      assert( 0 );  // Unknown segment type
    }
  }

  return false;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModuleEndMachine::FillRoutingTable()
{
  const bool bHasNic = GetOs()->GetHardwareInterface()->GetNumNics() > 0;
  const bool bHasRouter = GetRouterIp() != STR_NULL;
  CnlIPv4Address myIpAddress;
  EnlResult eResult = GetIpAddress( myIpAddress, 0 );

  if ( bHasNic && bHasRouter )
  {
    //2 For destinations outside my local network
    AddRowToRoutingTable(
      _T( "0.0.0.0" ),
      _T( "0.0.0.0" ),
      GetRouterIp(),
      myIpAddress.Machine());
  }
  
  //2 Internal loopback
  AddRowToRoutingTable(
    _T( "127.0.0.0" ),
    _T( "255.0.0.0" ),
    strInterfaceLoopBack,
    strInterfaceLoopBack );

  if ( bHasNic )
  {
    //2 For destinations inside my local network
    const CnlNetworkModel* const pNetworkModel =
      CnlSystemMgr::Instance().GetInternalInterface();
    const unsigned int nMask = 
      pNetworkModel->MakeMsbMask( myIpAddress.Length() );
    CString strMask;
    eResult = 
      pNetworkModel->NumericToDot( strMask, nMask, IP_VERSION_4 );
    
    AddRowToRoutingTable(
      myIpAddress.Network(),
      strMask,
      myIpAddress.Machine(),
      myIpAddress.Machine() );
  
  //2 Alternative loopback
    AddRowToRoutingTable(
      myIpAddress.Machine(),
      strIpBroadcastAddress,
      strInterfaceLoopBack,
      strInterfaceLoopBack );

  //2 Broadcast inside my local network
    AddRowToRoutingTable(
      myIpAddress.Broadcast(),
      strIpBroadcastAddress,
      myIpAddress.Machine(),
      myIpAddress.Machine() );
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModuleEndMachine::QueryForRouterIp()
{
  CnlIPv4Address address;
  GetIpAddress( address, 0 );
  const CnlPacket packet = MakePacket(
    address.Broadcast(),
    TRANSPORT_PROTOCOL_ROUTER_QUERY,
    CnlSegment() );

  SendPacket( 
    packet, 
    NETWORK_PROTOCOL_IP,
    true );
}


