/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlIpModuleRouter.h
//
// @short    See nlIpModuleRouter.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>
#include <nlCompositeHandle.h>

//1 Local
#include "nlIpModuleRouter.h"
#include "nlSystemMgrType.h"
#include "nlPacket.h"
#include "nlIPv4Address.h"
#include "nlOs.h"
#include "nlHwSwInterface.h"
#include "nlSegment.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::map< CString, unsigned int >::const_iterator IpNicMapConstIter;
  typedef std::set< CString >::const_iterator IPv4SetConstIter;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlIpModuleRouter::SnlImp* CnlIpModuleRouter::Imp()       { return m_pImp; }
inline const  CnlIpModuleRouter::SnlImp* CnlIpModuleRouter::Imp() const { return m_pImp; }

struct CnlIpModuleRouter::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlIpModuleRouter* const pOwner );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlIpModuleRouter* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The IP address assigned to the NIC in my machine that connects to a gateway router, if any
  CnlIPv4Address m_GatewayNicIpAddress;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlIpModuleRouter::SnlImp::SnlImp(
  CnlIpModuleRouter* const pOwner )
 :
  m_pOwner( pOwner )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlIpModuleRouter::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlIpModuleRouter::CnlIpModuleRouter(
  CnlOs* const pOs )
 :
  CnlIpModule( pOs )
{
  m_pImp = new SnlImp( this );
}

///////////////////////////////////////////////////////////////////////////////
CnlIpModuleRouter::~CnlIpModuleRouter()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModuleRouter::Initialize(
//  CnlArpModule* const pArpModule,
  CnlLinkLayerInterface* const pEthernetModule )
{
  CnlIpModule::Initialize( pEthernetModule );
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModuleRouter::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIpModuleRouter::ReceivePacket(
  const CnlPacket& packet )
{
  //2 Announce it
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  const CString strProtocol = 
    pNetworkModel->TransportProtocolToString( packet.GetTransportProtocol() );
  CString strMessage;
  strMessage.Format( 
    _T( "Receive IP packet: " )
    _T( "Transport Protocol: %s, " )
    _T( "SourceIp: %s, " )
    _T( "DestinationIp: %s" ),
    strProtocol,
    packet.GetSourceIpAddress(),
    packet.GetDestinationIpAddress() );

  pNetworkModel->SuspendAtPof( 
    GetOs()->GetHardwareInterface()->GetProcessingMachineHandle(), 
    LAYER_NETWORK,
    EVENT_TYPE_RECEIVE_ACCEPT,
    strMessage );
    
  if ( IsMyIpAddress( packet.GetDestinationIpAddress() ) )
  {
    //2 Packet sent to me explicitly
    const bool bHandled = CnlIpModule::ReceivePacket( packet );

    if ( bHandled )
    {
      return true;
    }

    //2 Special handing for messages sent to me
    //3 None
    switch( packet.GetTransportProtocol() )
    {
      case TRANSPORT_PROTOCOL_ROUTER_QUERY :
      {
        //2 Someone is looking for me; respond
        const CnlPacket responsePacket = MakePacket(
          packet.GetSourceIpAddress(),
          TRANSPORT_PROTOCOL_ROUTER_RESPONSE,
          CnlSegment() );

        SendPacket( 
          responsePacket, 
          NETWORK_PROTOCOL_IP,
          true );
        break;
      }
    }
    
  }
  else
  {
    //2 Packet sent to me for routing purposes
    SendPacket( 
      packet, 
      NETWORK_PROTOCOL_IP,
      true );
  }

  return false;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModuleRouter::FillRoutingTable()
{
  const bool bHasGatewayNic = Imp()->m_GatewayNicIpAddress.Machine() != STR_NULL;
  const bool bHasRouter = GetRouterIp()!= STR_NULL;

  if ( bHasGatewayNic && bHasRouter )
  {
    //2 For destinations outside my routing networks
    AddRowToRoutingTable(
      _T( "0.0.0.0" ),
      _T( "0.0.0.0" ),
      GetRouterIp(),
      Imp()->m_GatewayNicIpAddress.Machine());
  }
  
  //2 Internal loopback
  AddRowToRoutingTable(
    _T( "127.0.0.0" ),
    _T( "255.0.0.0" ),
    strInterfaceLoopBack,
    strInterfaceLoopBack );

  const unsigned int nNumRoutingNics = 
    GetOs()->GetHardwareInterface()->GetNumNics();

  for ( unsigned int nRoutingNic = 0; nRoutingNic < nNumRoutingNics; ++nRoutingNic )
  {
    //2 For destinations inside my routing networks
    CnlIPv4Address routingIpAddress;
    EnlResult eResult = GetIpAddress( routingIpAddress, nRoutingNic );
    const CnlNetworkModel* const pNetworkModel =
      CnlSystemMgr::Instance().GetInternalInterface();
    const unsigned int nMask = 
      pNetworkModel->MakeMsbMask( routingIpAddress.Length() );
    CString strMask;
    eResult = 
      pNetworkModel->NumericToDot( strMask, nMask, IP_VERSION_4 );
    
    AddRowToRoutingTable(
      routingIpAddress.Network(),
      strMask,
      routingIpAddress.Machine(),
      routingIpAddress.Machine() );

    //2 Alternative loopback
    AddRowToRoutingTable(
      routingIpAddress.Machine(),
      strIpBroadcastAddress,
      strInterfaceLoopBack,
      strInterfaceLoopBack );

    //2 Broadcast inside my local network
    AddRowToRoutingTable(
      routingIpAddress.Broadcast(),
      strIpBroadcastAddress,
      routingIpAddress.Machine(),
      routingIpAddress.Machine() );
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlIpModuleRouter::QueryForRouterIp()
{
  //2 Not supported yet
}

