/////////////////////////////////////////////////////////////////////////////
//
// @file     $Archive: /src/.../nlNetwork.cpp $
//
// @descr    See nlNetwork.h
//
// @short    See nlNetwork.h
//
// @version  $Revision: 0 $ $Date: xx/xx/2002 12:00p $ $Author: Jc5578 $
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCompositeHandle.h>
#include <nlIntegerPool.h>
#include <nlCommonType.h>

//1 Local
#include "nlNetwork.h"
#include "nlLink.h"
#include "nlProcessingMachine.h"
#include "nlNetworkModel.h"
#include "nlSystemMgr.h"
#include "nlUserPayload.h"
#include "nlIPv4Address.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlSystemMgrType;
using namespace NnlType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::map< unsigned long, CnlProcessingMachine* >::iterator MachineMapIter;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlNetwork::SnlImp* CnlNetwork::Imp()       { return m_pImp; }
inline const  CnlNetwork::SnlImp* CnlNetwork::Imp() const { return m_pImp; }

struct CnlNetwork::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlNetwork* const pOwner,
    CnlCompositeHandle hThis,
    const CnlIPv4Address& address );

  // Destructor
   ~SnlImp();

  // Create a new end machine in this network
  EnlResult CnlNetwork::SnlImp::MakeAndRegisterProcessingMachine(
    CnlCompositeHandle& hProcessingMachine,
    const CnlCompositeHandle hNetwork );

  // Get a pointer to the specified end machine
  NnlSystemMgrType::EnlResult GetProcessingMachine( 
    CnlProcessingMachine*& pProcessingMachine,
    CnlCompositeHandle hProcessingMachine ) const;

  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CnlNetwork* const m_pOwner;

  // Composite handle to the object that owns this implementation
  const CnlCompositeHandle m_hThis;
  
  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // My address
  const CnlIPv4Address m_Address;

  // The link that connects my end machines
  CnlLink* const m_pLink;

  // The IDs of the handles I'm using to identify my machines
  std::set< int > m_Handles;
  
  // The End Machines I connect
  std::map< unsigned long, CnlProcessingMachine* > m_ProcessingMachineHandleMap;
  
  // A pool of handle IDs
  CnlIntegerPool* const m_pHandleIdPool;

private:

  // Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlNetwork::SnlImp::SnlImp(
  CnlNetwork* const pOwner,
  CnlCompositeHandle hThis,
  const CnlIPv4Address& address )
 :
  m_pOwner( pOwner ),
  m_hThis( hThis ),
  m_Address( address ),
  m_pLink( new CnlLink( m_hThis ) ),
  m_pHandleIdPool( new CnlIntegerPool( nMaxNumProcessingMachines ) )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlNetwork::SnlImp::~SnlImp()
{
  //2 Delete the processing machines
  for ( MachineMapIter iter = m_ProcessingMachineHandleMap.begin();
        iter != m_ProcessingMachineHandleMap.end();
        ++iter )
  {
    delete ( *iter ).second;
  }

  
  delete m_pHandleIdPool;
  delete m_pLink;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::SnlImp::MakeAndRegisterProcessingMachine(
  CnlCompositeHandle& hProcessingMachine,
  const CnlCompositeHandle hNetwork )
{
  //1 See if we can create another end machine
  if ( ! m_pHandleIdPool->IsAvailable() )
  {
    //2 Maximun number of networks already exists
    return RESULT_ERROR_MAX_EXCEEDED;
  }

  //1 Create and register the end machine
  const unsigned long nHandleID = m_pHandleIdPool->CheckOut();
  hProcessingMachine = CnlCompositeHandle( 
    HANDLE_TYPE_END_MACHINE, 
    hNetwork, 
    nHandleID );
  CnlProcessingMachine* pProcessingMachine = 
    new CnlProcessingMachine( hProcessingMachine, MACHINE_TYPE_END_MACHINE );
  pProcessingMachine->Initialize();
  m_ProcessingMachineHandleMap[ nHandleID ] = pProcessingMachine;

  return RESULT_OK;  
}

#if 0
///////////////////////////////////////////////////////////////////////////////
void CnlNetwork::SnlImp::DestroyProcessingMachine(
  CnlCompositeHandle hProcessingMachine )
{
  //!!!jc todo
  // Find the entry in m_NetworkHandleMap and remove it
}
#endif

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::SnlImp::GetProcessingMachine( 
  CnlProcessingMachine*& pProcessingMachine,
  CnlCompositeHandle hProcessingMachine ) const
{
  //1 Try to locate this machine
  std::map< unsigned long, CnlProcessingMachine* >::const_iterator iter = 
    m_ProcessingMachineHandleMap.find( hProcessingMachine.ID() );
  
  if ( iter == m_ProcessingMachineHandleMap.end() )
  {
    return RESULT_ERROR_END_MACHINE_DOES_NOT_EXIST;
  }

  pProcessingMachine = ( *iter ).second;
  return RESULT_OK;
  
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlNetwork::CnlNetwork(
  CnlCompositeHandle hThis,
  const CnlIPv4Address& address )
{
  m_pImp = new SnlImp( this, hThis, address );
}

///////////////////////////////////////////////////////////////////////////////
CnlNetwork::~CnlNetwork()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlNetwork::Initialize()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlNetwork::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::CreateProcessingMachine(
  CnlCompositeHandle& hProcessingMachine,
  CnlCompositeHandle hNetwork )
{
  //1 See if we can fit another machine
  EnlResult eResult = RESULT_UNKNOWN;
  eResult = Imp()->MakeAndRegisterProcessingMachine( hProcessingMachine, hNetwork );

  if ( eResult != RESULT_OK )
  {
    return eResult;
  }
  
  //1 Connect the Nic to the link
  CnlProcessingMachine* pProcessingMachine = NULL;
  eResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );
  assert( eResult == RESULT_OK );

  CnlNic* const pNic = pProcessingMachine->GetNic( 0 );

  Imp()->m_pLink->ConnectNic( pNic );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::ConnectRouter(
  const CnlProcessingMachine* pRouter,
  const unsigned int nNicId )
{
  CnlNic* const pNic = pRouter->GetNic( nNicId );
  assert( pNic != NULL );

  Imp()->m_pLink->ConnectNic( pNic );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::AssignIpAddress(
  const CnlCompositeHandle hProcessingMachine,
  const unsigned int nNicId,
  const CString& strClasslessIpAddress )
{
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->AssignIpAddress( nNicId, strClasslessIpAddress );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::ExecuteCommand(
  const CnlCompositeHandle hProcessingMachine,
  const CnlCommand* const pCommand )
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->ExecuteCommand( pCommand );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::PostExecuteCommand(
  const CnlCompositeHandle hProcessingMachine,
  const CnlCommand* const pCommand )
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->PostExecuteCommand( pCommand );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::GetProcessingMachineIpAddress(
  CString& strIpAddress,
  const CnlCompositeHandle hProcessingMachine,
  const unsigned int nNicOffset ) const
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->GetIpAddress( strIpAddress, nNicOffset );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::GetProcessingMachineMacAddress(
  CString& strMacAddress,
  const CnlCompositeHandle hProcessingMachine,
  const unsigned int nNicOffset ) const
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->GetMacAddress( strMacAddress, nNicOffset );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::GetProcessingMachineNicCount(
  unsigned int& nNumNics,
  const CnlCompositeHandle hProcessingMachine ) const
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->GetNicCount( nNumNics );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::GetProcessingMachineString(
  CString& strConsoleOutput,
  const CnlCompositeHandle hProcessingMachine,
  NnlSystemMgrType::EnlCommandLineEntry eEntry ) const
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->GetString( strConsoleOutput, eEntry );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::ToggleProcessingMachinePower(
  const CnlCompositeHandle hProcessingMachine )
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->TogglePower();
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetwork::GetProcessingMachinePowerStatus(
  bool& bIsOn,
  const CnlCompositeHandle hProcessingMachine ) const
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->GetPowerStatus( bIsOn );
}


///////////////////////////////////////////////////////////////////////////////
NnlSystemMgrType::EnlResult CnlNetwork::ClearProcessingMachineArpCache(
  const CnlCompositeHandle hProcessingMachine )
{
  //1 Error checking
  CnlProcessingMachine* pProcessingMachine = NULL;
  EnlResult nResult = Imp()->GetProcessingMachine( pProcessingMachine, hProcessingMachine );

  if ( nResult != RESULT_OK )
  {
    //3 Problem accessing specified end machine
    return nResult;
  }

  //2 Found the end machine; hand it off for processing
  return pProcessingMachine->ClearArpCache();
}

