//////////////////////////////////////////////////////////////////////////////
//
// @descr    Interface for Network Layer modules
//
// @short    Interface for Network Layer modules
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_NETWORK_LAYER_INTERFACE_H__
#define __NL_NETWORK_LAYER_INTERFACE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlTransportLayerInterface;
class CnlSegment;
class CnlLinkLayerInterface;
class CnlPacket;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlNetworkLayerInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlNetworkLayerInterface()
    {}

    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlNetworkLayerInterface()
    {}

    //1 Abstract
    // @descr   Pass the specified payload down for broadcast in the network
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload passed down in a segment
    // @param   eSegmentType  IN: Type of link-layer segment
    // @param   segment  IN: Payload to deliver to Link layer
    // @end
//    virtual void Broadcast(
//      const NnlSystemMgrType::EnlSegmentProtocol eSegmentType,
//      const CnlSegment& segment ) = 0;

    // @descr   Send the specified payload to the specified IP Address
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload sent
    // @param   eSegmentType  IN: Type of link-layer segment
    // @param   segment  IN: Payload to deliver to Link layer
    // @param   strIpAddress  IN: Address to send to
    // @end
//    virtual void SendToIpAddress(
//      const NnlSystemMgrType::EnlSegmentProtocol eSegmentType,
//      const CnlSegment& segment,
//      const CString strIpAddress ) = 0 ;

    // @descr   Receive a packet from the link layer module.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Packet processed
    // @param   frame  IN: Packet to process.
    // @return  True if the packet was handled; False if discarded
    // @end
    virtual bool ReceivePacket(
      const CnlPacket& packet ) = 0;

    virtual void ReceiveArp(
      CnlPacket payload,
      const CString& strRequestingMachineMac ) = 0;

    // @descr   Receive a RARP command from the network
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Modify the ARP cache
    // @param   payload  IN: RARP payload
    // @end
    virtual void ReceiveRarp(
      CnlPacket payload,
      const CString& strRequestingMachineMac ) = 0;

    // @descr   Does this IP address belong to this machine?
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Result returned.
    // @param   strIpAddress  IN: Address to check
    // @return  True if any Nic card is assigned this address; False otherwise
    // @end
    virtual bool IsMyIpAddress(
      const CString& strIpAddress ) const = 0;

    // @descr   Lookup an entry in the routing table
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Values corresponding to the best match are output
    // @param   strGateway  OUT: "Gateway" column value
    // @param   strInterface  OUT: "Interface" column value
    // @param   strIpToLookup  IN: IP address to lookup
    // @return  True if a match found; False otherwise
    // @end
    virtual bool Lookup( 
      CString& strGateway,
      CString& strInterface,
      const CString& strIpToLookup ) const = 0;

//    virtual unsigned int GetNicOffsetFromInterface( 
//      const CString& strInterface ) const = 0;

  protected:

  private:

    //1 Implementation
    //3 None
    
    //1 Unimplemented
    CnlNetworkLayerInterface( const CnlNetworkLayerInterface& );
    CnlNetworkLayerInterface& operator=( const CnlNetworkLayerInterface& );
    CnlNetworkLayerInterface* operator&();
    const CnlNetworkLayerInterface* operator&() const;
};

#endif

