/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlNetworkModel.h
//
// @short    See nlNetworkModel.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

#include "nlNetworkModelImp.h"
//1 External
//2 From Utilities
//#include <agMfcWorkerThread.h>
#include <nlMfcThread.h>
#include <nlThreadHandler.h>
#include <nlCommonType.h>
#include <nlModel.h>
#include <nlCompositeHandle.h>

//1 Local
#include "nlNetworkModel.h"
#include "nlNetworkModelImp.h"
#include "nlNetwork.h"
#include <nlIntegerPool.h>
#include "nlCommand.h"
#include "nlIPv4Address.h"
#include "nlProcessingMachine.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlSystemMgrType;
using namespace NnlType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::map< unsigned long, CnlNetwork* >::iterator NetworkMapIter;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Worker thread classes
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
class CnlExecuteCommandThread : public CnlMfcThread
{
  public:

    // @descr  Constructor
    // @param  pNetworkModel  IN: Handle to the Network Model
    // @end
    CnlExecuteCommandThread(
      CnlNetworkModel* const pNetworkModel,
      const CnlCompositeHandle hProcessingMachine,
      const CnlCommand* const pCommand );
};

class CnlExecuteCommandThreadHandler : public CnlThreadHandler
{
  public:

    // @descr  Constructor
    // @param  pNetworkModel  IN: Handle to the Network Model
    // @end
    CnlExecuteCommandThreadHandler(
      CnlNetworkModel* const pNetworkModel,
      const CnlCompositeHandle hProcessingMachine,
      const CnlCommand* const pCommand );

    // @descr   Called when the threads starts executing.
    // @return  
    // @end
    //
    unsigned __int32 ThreadHandlerProc();

  private:

    // Handle to the Network Model
    CnlNetworkModel* const m_pNetworkModel;
    const CnlCompositeHandle m_hProcessingMachine;
    const CnlCommand* const m_pCommand;
};

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlNetworkModel::SnlImp* CnlNetworkModel::Imp()       { return m_pImp; }
inline const  CnlNetworkModel::SnlImp* CnlNetworkModel::Imp() const { return m_pImp; }

//1 See nlNetworkModelImp.h for Insulated data declarations

//1Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlNetworkModel::SnlImp::SnlImp(
  CnlNetworkModel* const pOwner )
 :
  m_pOwner( pOwner ),
  m_pHandleIdPool( new CnlIntegerPool( nMaxNumNetworks ) ),
  m_pExecuteCommandThread( NULL ),
  m_eLowestLayerToShow( LAYER_NETWORK ),
  m_nPofSequenceNum( 0 ),
  m_eThreadStatus( THREAD_STATUS_COMPLETED ),
  m_bPofEnabled( false )
{
  assert( NUM_LAYERS_PLUS_1 == 6 );
  m_LayerStrings.push_back( STR_NULL );
  m_LayerStrings.push_back( _T("Physical" ) );
  m_LayerStrings.push_back( _T("Link" ) );
  m_LayerStrings.push_back( _T("Network" ) );
  m_LayerStrings.push_back( _T("Transport" ) );
  m_LayerStrings.push_back( _T("Upper" ) );
  m_LayerStrings.push_back( STR_NULL );

  m_ShowEventSettings.push_back( false );  // EVENT_TYPE_UNKNOWN
  m_ShowEventSettings.push_back( true );  // EVENT_TYPE_INFO
  m_ShowEventSettings.push_back( true );  // EVENT_TYPE_SEND
  m_ShowEventSettings.push_back( true );  // EVENT_TYPE_RECEIVE_ACCEPT
  m_ShowEventSettings.push_back( false );  // EVENT_TYPE_RECEIVE_REJECT
  m_ShowEventSettings.push_back( false );  // NUM_EVENT_TYPES
}

///////////////////////////////////////////////////////////////////////////////
CnlNetworkModel::SnlImp::~SnlImp()
{
  //2 Delete the networks
  for ( NetworkMapIter iter = m_NetworkHandleMap.begin();
        iter != m_NetworkHandleMap.end();
        ++iter )
  {
    delete ( *iter ).second;
  }

  delete m_pExecuteCommandThread;
  delete m_pHandleIdPool;
}
 
///////////////////////////////////////////////////////////////////////////////
CnlExecuteCommandThread::CnlExecuteCommandThread( 
  CnlNetworkModel* const pNetworkModel,
  const CnlCompositeHandle hProcessingMachine,
  const CnlCommand* const pCommand )
 :
  CnlMfcThread(  new CnlExecuteCommandThreadHandler(
    pNetworkModel,
    hProcessingMachine,
    pCommand ),
    CREATE_SUSPENDED )
{
  pNetworkModel->Imp()->m_hThisThread = GetHandle();
  SetThreadPriority( GetHandle(), THREAD_PRIORITY_BELOW_NORMAL );
//  ::ResumeThread( GetHandle() );
}

///////////////////////////////////////////////////////////////////////////////
CnlExecuteCommandThreadHandler::CnlExecuteCommandThreadHandler(
  CnlNetworkModel* const pNetworkModel,
  const CnlCompositeHandle hProcessingMachine,
  const CnlCommand* const pCommand )
 : 
  m_pNetworkModel( pNetworkModel ),
  m_hProcessingMachine( hProcessingMachine ),
  m_pCommand( pCommand )
{
}

///////////////////////////////////////////////////////////////////////////////
unsigned __int32 CnlExecuteCommandThreadHandler::ThreadHandlerProc()
{
  m_pNetworkModel->Imp()->m_eResultFromThread = 
    m_pNetworkModel->Imp()->ExecuteCommand( m_hProcessingMachine, m_pCommand );
  m_pNetworkModel->Imp()->PostExecuteCommand( m_hProcessingMachine, m_pCommand );
  delete m_pCommand;
  TRACE0( _T( "\n=====END OF COMMAND=====\n" ) );
  m_pNetworkModel->Imp()->m_eThreadStatus = THREAD_STATUS_COMPLETED;
  return 0;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlNetworkModel::SnlImp::IsNumProcessingMachinesValidForSubnet( 
  const int nSubnetAddressLength,
  const int nNumProcessingMachines )
{
  const int nMaxNumAddresses = 
    ( 1 << ( nNumBitsInIPv4Address - nSubnetAddressLength ) );
  const int nMaxNumProcessingMachines = 
    nMaxNumAddresses  - nNumReservedAddressesPerSubnet;

  //3 Policy: disallow empty networks
  return ( ( nNumProcessingMachines > 0 ) && (  nNumProcessingMachines <= nMaxNumProcessingMachines ) );
}
  
///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::SnlImp::GetNetwork(
  CnlNetwork*& pNetwork,
  const CnlCompositeHandle hNetwork ) const
{
  //1 Try to locate this network in the internet
  if ( ! hNetwork.IsValid() )
  {
    //2 This handle can't identify the owning network
    return RESULT_ERROR_INVALID_HANDLE;
  }

  std::map< unsigned long, CnlNetwork* >::const_iterator iter = 
    m_NetworkHandleMap.find( hNetwork.ID() );
  
  if ( iter == m_NetworkHandleMap.end() )
  {
    return RESULT_ERROR_NETWORK_DOES_NOT_EXIST;
  }

  pNetwork = ( *iter ).second;
  return RESULT_OK;
}
  
///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::SnlImp::MakeAndRegisterNetwork(
  CnlCompositeHandle& hNetworkOut,
  const CnlIPv4Address& address )
{
  //1 See if we can create another network
  if ( ! m_pHandleIdPool->IsAvailable() )
  {
    //2 Maximun number of networks already exists
    return RESULT_ERROR_MAX_EXCEEDED;
  }

  //1 Create and register the network
  const unsigned long nHandleID = m_pHandleIdPool->CheckOut();
  CnlCompositeHandle hNetwork = CnlCompositeHandle( 
    HANDLE_TYPE_NETWORK,
    nHandleID );
  CnlNetwork* pNetwork = new CnlNetwork( hNetwork, address );
  m_NetworkAddressRegistry.insert( address.Network() );
  m_NetworkHandleMap[ nHandleID ] = pNetwork;

  //1 Set outputs
  hNetworkOut = hNetwork;
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
void CnlNetworkModel::SnlImp::DestroyNetwork(
  CnlCompositeHandle )
{
  //!!!jc todo
  // Find the entry in m_NetworkHandleMap and remove it
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::SnlImp::ExecuteCommand(
  const CnlCompositeHandle hProcessingMachine,
  const CnlCommand* const pCommand )
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;

      eResult = GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 Problem accessing specified network
        return eResult;
      }

      //2 Found the network that owns it, hand it off for processing
      return pNetwork->ExecuteCommand( hProcessingMachine, pCommand );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return m_pRouter->ExecuteCommand( pCommand );
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}


///////////////////////////////////////////////////////////////////////////////
void CnlNetworkModel::SnlImp:: PostExecuteCommand(
  const CnlCompositeHandle hProcessingMachine,
  const CnlCommand* const pCommand )
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;

      eResult = GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 Problem accessing specified network
        return;
      }

      //2 Found the network that owns it, hand it off for processing
      pNetwork->PostExecuteCommand( hProcessingMachine, pCommand );
      return;
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      m_pRouter->PostExecuteCommand( pCommand );
      return;
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
    }
    default : assert( 0 );  // Don't know this type
  }
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::SnlImp:: MakeRandomChar()
{
  int nMaxValue = 16;
  int n = int ( ( double( rand() ) / RAND_MAX ) * nMaxValue );
  CString str;
  str.Format( _T( "%x" ), n);
  return str;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlNetworkModel::SnlImp::IsAddressUnique( 
  const CString& strAddress ) const
{
  std::set< CString >::const_iterator iter = m_NetworkAddressRegistry.find( strAddress );
  return ( iter == m_NetworkAddressRegistry.end() );
}

///////////////////////////////////////////////////////////////////////////////
bool CnlNetworkModel::SnlImp::ValidateAddressLength(
  const int nLength ) const
{
  if ( ( nLength < nNumBitsInIPv4Address - nMaxNumHostBits )
    || ( nLength > nNumBitsInIPv4Address - nMinNumHostBits ) )
  {
    return false;
  }

  return true;
}


///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlNetworkModel::CnlNetworkModel()
{
  m_pImp = new SnlImp( this );
}

///////////////////////////////////////////////////////////////////////////////
CnlNetworkModel::~CnlNetworkModel()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlNetworkModel::Initialize()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlNetworkModel::PreDestroy()
{
  UnregisterAllObservers();
}

#if 0
///////////////////////////////////////////////////////////////////////////////
bool CnlNetworkModel::UnitTest()
{
  return Imp()->UnitTest();
}
#endif

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::AttachObserver(
  CnlObserver* const pObserver )
{
  CnlModel::AttachObserver( pObserver );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::DetachObserver(
  CnlObserver* const pObserver )
{
  CnlModel::DetachObserver( pObserver );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
void CnlNetworkModel::NotifyObserversOfChange(
  NnlSystemMgrType::EnlMessage eMessage )
{
  NotifyObservers( eMessage );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::AddNetworkToInternet(
  CnlCompositeHandle& hNetworkOut,
  const CString& strClasslessFormat )
{
  CnlIPv4Address address( strClasslessFormat );
  
  //1 Error checking
  if ( address.Validity() != RESULT_OK )
  {
    //3 Invalid address
    return address.Validity();
  }

  if ( ! Imp()->IsAddressUnique( address.Network() ) )
  {
    return RESULT_ERROR_DUPLICATE_ADDRESS;
  }

  if ( ! Imp()->ValidateAddressLength( address.Length() ) )
  {
    return RESULT_ERROR_INVALID_ADDRESS_LENGTH;
  }
  
  CnlCompositeHandle hNetwork;
  const EnlResult eResult = Imp()->MakeAndRegisterNetwork(
    hNetwork,
    address );

  if ( eResult !=  RESULT_OK )
  {
    return eResult;
  }
  else
  {
    hNetworkOut = hNetwork;
    return RESULT_OK;
  }
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::AddProcessingMachineToNetwork(
  CnlCompositeHandle& hProcessingMachineOut,
  const CnlCompositeHandle hNetwork )
{
  CnlNetwork* pNetwork = NULL;
  
  EnlResult eResult = Imp()->GetNetwork( pNetwork, hNetwork );

  if ( eResult != RESULT_OK )
  {
    return eResult;
  }

  CnlCompositeHandle hProcessingMachine;
  eResult = pNetwork->CreateProcessingMachine( hProcessingMachine, hNetwork );

  if ( eResult != RESULT_OK )
  {
    return eResult;
  }

  hProcessingMachineOut = hProcessingMachine;
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::MakeRouter(
  CnlCompositeHandle& hRouterOut )
{
  //1 Create and register the network
  const unsigned long nHandleID = Imp()->m_pHandleIdPool->CheckOut();
  CnlCompositeHandle hRouter = CnlCompositeHandle(
    HANDLE_TYPE_ROUTER,
    nHandleID );
  CnlProcessingMachine* pRouter = new CnlProcessingMachine(
    hRouter,
    MACHINE_TYPE_ROUTER );
  pRouter->Initialize();
  Imp()->m_pRouter = pRouter;
//  m_NetworkHandleMap[ nHandleID ] = pNetwork;

  //1 Set outputs
  hRouterOut = hRouter;
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::SuspendAtPof(
  const CnlCompositeHandle hPof,
  const NnlSystemMgrType::EnlLayer eLayer,
  const EnlEventType eEventType,
  const CString& strMessage )
{
TRACE2( _T("\nSAP Layer %d Event %d" ), eLayer, eEventType );
  //1 Error checking
  if ( ! IsCommandExecuting() )
  {
    //1 An internal operation took place
    return RESULT_OK;
  }
  
  //1 Check filters
  if ( ! Imp()->m_bPofEnabled )
  {
    //2 The view is probably still initializing
    return RESULT_POF_PROCESSING_DISABLED;
  }
  
  if ( eLayer < Imp()->m_eLowestLayerToShow )
  {
    return RESULT_LAYER_FILTERED_OUT;
  }

  if ( ! Imp()->m_ShowEventSettings[ eEventType ] )
  {
    return RESULT_EVENT_TYPE_FILTERED_OUT;
  }

  //1 Do it
  CString strLayerId; 

  switch ( eLayer )
  {
    case LAYER_PHYSICAL :
    {
      strLayerId = _T( "<physical>" );
      break;
    }
    case LAYER_LINK:
    {
      strLayerId = _T( "<link>" );
      break;
    }
    case LAYER_NETWORK:
    {
      strLayerId = _T( "<network>" );
      break;
    }
    case LAYER_TRANSPORT:
    {
      strLayerId = _T( "<transport>" );
      break;
    }
    case LAYER_UPPER :
    {
      strLayerId = _T( "<upper>" );
      break;
    }
  }
  
  Imp()->m_hPof = hPof;
  ++Imp()->m_nPofSequenceNum;
  Imp()->m_strPofMessage.Format(
    _T("%d) %s %s" ), Imp()->m_nPofSequenceNum, strLayerId, strMessage );
  TRACE1( _T( "\nPOF %s" ), Imp()->m_strPofMessage );

  Imp()->m_eMessageFromThread = MESSAGE_POF_CHANGED;
  Imp()->m_eThreadStatus = THREAD_STATUS_SUSPENDED;
  ::SuspendThread( Imp()->m_hThisThread );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle CnlNetworkModel::GetPofHandle()
{
  return Imp()->m_hPof;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::GetPofMessage()
{
  return Imp()->m_strPofMessage;
}

///////////////////////////////////////////////////////////////////////////////
unsigned int CnlNetworkModel::GetPofSequenceNum() const
{
  return Imp()->m_nPofSequenceNum;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::AssignIpAddress(
  const CnlCompositeHandle hProcessingMachine,
  const unsigned int nNicId,
  const CString& strClasslessIpAddress )
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;
      eResult = Imp()->GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 No such network
        return eResult;
      }

      //2 Found the network that owns it, hand it off for processing
      return pNetwork->AssignIpAddress( hProcessingMachine, nNicId, strClasslessIpAddress );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->AssignIpAddress( nNicId, strClasslessIpAddress );
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::ConnectRouter(
  const CnlCompositeHandle hNetwork,
  const unsigned int nNicId )
{
  if ( hNetwork.Type() != HANDLE_TYPE_NETWORK )
  {
    //2 Expected a network handle; got some other kind
    return RESULT_ERROR_WRONG_HANDLE_TYPE;
  }
  
  CnlNetwork* pNetwork = NULL;
  const EnlResult eResult = Imp()->GetNetwork( pNetwork, hNetwork );

  if ( eResult != RESULT_OK )
  {
    //3 No such network
    return eResult;
  }

  //2 Found the network that owns it, hand it off for processing
  return pNetwork->ConnectRouter( Imp()->m_pRouter, nNicId );
}

///////////////////////////////////////////////////////////////////////////////
const CnlCommand* CnlNetworkModel::MakeCommandArp(
  const CString& strWhoHasIpAddress ) const
{
  CnlCommand* pCommand = new CnlCommand( COMMAND_ARP );
  pCommand->AddStringParameter( strWhoHasIpAddress );
  return pCommand;
}

///////////////////////////////////////////////////////////////////////////////
const CnlCommand* CnlNetworkModel::MakeCommandPing(
  const CString& strPingTargetAddress ) const
{
  CnlCommand* pCommand = new CnlCommand( COMMAND_PING );
  pCommand->AddStringParameter( strPingTargetAddress );
  return pCommand;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::ExecuteCommand(
  const CnlCompositeHandle hProcessingMachine,
  const CnlCommand* const pCommand )
{
  pCommand->Dump();
  
  if ( IsCommandExecuting() )
  {
    //2 The previous execution is incomplete; disallow simultaneous commands
    return RESULT_ERROR_PREVIOUS_EXECUTION_IMCOMPLETE;
  }

  TRACE0( _T( "\n=====START OF COMMAND=====\n" ) );
  Imp()->m_eResultFromThread = RESULT_EXECUTION_INCOMPLETE;
  Imp()->m_nPofSequenceNum = 0;
  NotifyObservers( MESSAGE_COMMAND_STARTED );

  //1 Spawn a thread to do this task
  delete Imp()->m_pExecuteCommandThread;
  Imp()->m_pExecuteCommandThread = new CnlExecuteCommandThread(
    this, hProcessingMachine, pCommand );  // Created suspended
  Imp()->m_eThreadStatus = THREAD_STATUS_RUNNING;

  return SingleStep();
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::SingleStep()
{
  if ( ! IsCommandExecuting() )
  {
    //2 There is no command currently executing
    return RESULT_ERROR_NO_COMMAND_EXECUTING;
  }
  
  //2 Wake the command execution thread up. It will run either until it completes
  //3 or until it puts itself to sleep
  //3 Note: The WaitFor command doesn't seem to work, I am using it here as a timer 
  //3 and using the member variable for thread status
  ::ResumeThread( Imp()->m_pExecuteCommandThread->GetHandle() );
  Imp()->m_eThreadStatus = THREAD_STATUS_RUNNING;

  TRACE1( _T("\nTS: %d" ), Imp()->m_eThreadStatus  );
  while ( Imp()->m_eThreadStatus == THREAD_STATUS_RUNNING )
  {
    ::WaitForSingleObject( Imp()->m_pExecuteCommandThread->GetHandle(), 10 );
  }
  TRACE1( _T("\nTS2: %d" ), Imp()->m_eThreadStatus  );

  //2 Thread suspended or complete
TRACE0( _T("\nSS: Thread suspended or complete" ) );
  if ( Imp()->m_eThreadStatus == THREAD_STATUS_COMPLETED )
  {
    Imp()->m_nPofSequenceNum = 0;
    NotifyObservers( MESSAGE_COMMAND_COMPLETED );
  }

  NotifyObservers( MESSAGE_POF_CHANGED );

  return Imp()->m_eResultFromThread;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::RunToEnd()
{
  if ( ! IsCommandExecuting() )
  {
    //2 There is no command currently executing
    return RESULT_ERROR_NO_COMMAND_EXECUTING;
  }
  
  //1 While task remains to be completed, restart the thread
  while ( Imp()->m_eThreadStatus != THREAD_STATUS_COMPLETED )
  {
    SingleStep();
  }

  return Imp()->m_eResultFromThread;
}

void CnlNetworkModel::Test()
{
  CnlIPv4Address address( _T( "0.0.0.0/16" ) );
  assert( address.Validity() == RESULT_OK );
  assert( address.Classless() == _T( "0.0.0.0/16" ) );
  assert( address.Machine() == _T( "0.0.0.0" ) );
  assert( address.Network() == _T( "0.0.0.0" ) );
  assert( address.Broadcast() == _T( "0.0.255.255" ) );
  assert( address.Gateway() == _T( "0.0.255.254" ) );
  assert( address.Length() == 16 );

  CnlIPv4Address address2( _T( "0.0.0.1/16" ) );
  assert( address2.Validity() == RESULT_OK );
  assert( address2.Classless() == _T( "0.0.0.1/16" ) );
  assert( address2.Machine() == _T( "0.0.0.1" ) );
  assert( address2.Network() == _T( "0.0.0.0" ) );
  assert( address2.Broadcast() == _T( "0.0.255.255" ) );
  assert( address2.Gateway() == _T( "0.0.255.254" ) );
  assert( address2.Length() == 16 );

  CnlIPv4Address address3( _T( "0.2.0.1/16" ) );
  assert( address3.Validity() == RESULT_OK );
  assert( address3.Classless() == _T( "0.2.0.1/16" ) );
  assert( address3.Machine() == _T( "0.2.0.1" ) );
  assert( address3.Network() == _T( "0.2.0.0" ) );
  assert( address3.Broadcast() == _T( "0.2.255.255" ) );
  assert( address3.Gateway() == _T( "0.2.255.254" ) );
  assert( address3.Length() == 16 );


//  CnlVariableParmContainer vpc;
  CnlCommand vpc( COMMAND_ARP );
  vpc.AddBoolParameter( true );
  vpc.AddBoolParameter( false );
  vpc.AddBoolParameter( true );

  vpc.AddNumericParameter( 100 );
  vpc.AddNumericParameter( 101 );
  vpc.AddNumericParameter( 102 );
  vpc.AddNumericParameter( 103 );

  vpc.AddStringParameter( _T( "A" ) );
  vpc.AddStringParameter( _T( "B" ) );
  vpc.AddStringParameter( _T( "C" ) );
  vpc.AddStringParameter( _T( "D" ) );
  vpc.AddStringParameter( _T( "E" ) );

  assert( vpc.GetType() == COMMAND_ARP );
  assert( vpc.GetNumBoolParameters() == 3 );
  assert( vpc.GetNumNumericParameters() == 4 );
  assert( vpc.GetNumStringParameters() == 5 );

  assert( vpc.GetBoolParameter( 0 ) == true );
  assert( vpc.GetBoolParameter( 1 ) == false );
  assert( vpc.GetBoolParameter( 2 ) == true );

  assert( vpc.GetNumericParameter( 0 ) == 100 );
  assert( vpc.GetNumericParameter( 1 ) == 101 );
  assert( vpc.GetNumericParameter( 2 ) == 102 );
  assert( vpc.GetNumericParameter( 3 ) == 103 );

  assert( vpc.GetStringParameter( 0 ) == _T( "A" ) );
  assert( vpc.GetStringParameter( 1 ) == _T( "B" ) );
  assert( vpc.GetStringParameter( 2 ) == _T( "C" ) );
  assert( vpc.GetStringParameter( 3 ) == _T( "D" ) );
  assert( vpc.GetStringParameter( 4 ) == _T( "E" ) );

  CnlVariableParmContainer vpc2 = vpc;
  assert( vpc2.GetNumBoolParameters() == 3 );
  assert( vpc2.GetNumNumericParameters() == 4 );
  assert( vpc2.GetNumStringParameters() == 5 );
  assert( vpc.GetBoolParameter( 2 ) == true );
  assert( vpc.GetNumericParameter( 3 ) == 103 );
  assert( vpc.GetStringParameter( 4 ) == _T( "E" ) );

  unsigned int nMask;
  CString strMask;

  nMask = MakeMsbMask( 0 );
  NumericToDot( strMask, nMask, IP_VERSION_4 );
  assert( strMask == _T( "0.0.0.0" )  );

  nMask = MakeMsbMask( 8 );
  NumericToDot( strMask, nMask, IP_VERSION_4 );
  assert( strMask == _T( "255.0.0.0" )  );

  nMask = MakeMsbMask( 16 );
  NumericToDot( strMask, nMask, IP_VERSION_4 );
  assert( strMask == _T( "255.255.0.0" )  );

  nMask = MakeMsbMask( 24 );
  NumericToDot( strMask, nMask, IP_VERSION_4 );
  assert( strMask == _T( "255.255.255.0" )  );

  nMask = MakeMsbMask( 32 );
  NumericToDot( strMask, nMask, IP_VERSION_4 );
  assert( strMask == _T( "255.255.255.255" )  );

  nMask = MakeMsbMask( 33 );
  NumericToDot( strMask, nMask, IP_VERSION_4 );
  assert( strMask == _T( "0.0.0.0" )  );

  int nBreakHere = 0;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::MakeMacAddress()
{
  //1 Generate a random address
  CString strMacAddress;
  strMacAddress.Format( 
    _T( "%s%s-%s%s-%s%s-%s%s-%s%s-%s%s" ),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar(),
    Imp()->MakeRandomChar() );
  
  //1 Make sure it is unique
  std::set< CString >::const_iterator iter = Imp()->m_MacAddressRegistry.find( strMacAddress );

  while ( iter != Imp()->m_MacAddressRegistry.end() )
  {
    //2 Duplicate; regenerate
    strMacAddress = MakeMacAddress();
    iter = Imp()->m_MacAddressRegistry.find( strMacAddress );
  }


  Imp()->m_MacAddressRegistry.insert( strMacAddress );
  return strMacAddress;
}

#include "TCHAR.H"
///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::DotToNumeric(
  unsigned long& nAddressOut,
  const CString& strDotNotation,
  NnlSystemMgrType::EnlIpVersion eIpVersion ) const
{
  assert( eIpVersion == IP_VERSION_4 );

  if ( ! IsValidDotNotation( strDotNotation, eIpVersion ) )
  {
    return RESULT_ERROR_INVALID_ADDRESS;
  }
  
  unsigned long nAddress = 0;
  CString strAddress = strDotNotation;
  LPTSTR pstrAddress = strAddress.GetBuffer( 0 );
  TCHAR pcAddressSeparator[] = _T( "." );

  //2 Get the first segment
  CString strAddressSegment = _tcstok( pstrAddress, pcAddressSeparator );

  for ( int nSegment = 0; nSegment < 4; ++nSegment )
  {
    const int nAddressValue = atoi( strAddressSegment );
    const int nShift = ( 3 - nSegment ) * 8;
    nAddress |= ( nAddressValue << nShift );

    //2 Get the next segment
    strAddressSegment = _tcstok( NULL, pcAddressSeparator );
  }

  strAddress.ReleaseBuffer( -1 );

  nAddressOut = nAddress;
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::NumericToDot(
  CString& strDotNotationOut,
  const unsigned long nAddress,
  NnlSystemMgrType::EnlIpVersion eIpVersion ) const
{
  assert( eIpVersion == IP_VERSION_4 );

  CString strDotNotation;
  TCHAR pcSegment[4];
  
  for ( int nSegment = 3; nSegment >= 0; --nSegment )
  {
    const int nSegmentValue = ( nAddress >> ( nSegment * 8 ) ) & 0xFF;

    if ( nSegmentValue < 0 || nSegmentValue > 255 )
    {
      return RESULT_ERROR_INVALID_ADDRESS;
    }

    strDotNotation += CString( itoa( nSegmentValue, pcSegment, 10 ) );

    if ( nSegment > 0 )
    {
      strDotNotation += _T( "." );
    }
  }

  strDotNotationOut = strDotNotation;
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlNetworkModel::IsValidDotNotation(
  const CString& strDotNotation,
  NnlSystemMgrType::EnlIpVersion eIpVersion ) const
{
  assert( eIpVersion == IP_VERSION_4 );
  
  CString strAddress = strDotNotation;
  LPTSTR pstrAddress = strAddress.GetBuffer( 0 );
  TCHAR pcAddressSeparator[] = _T( "." );

  //2 Get the first segment
  CString strAddressSegment = _tcstok( pstrAddress, pcAddressSeparator );

  for ( int nSegment = 0; nSegment < 4; ++nSegment )
  {
    //2 Validate the segment
    if ( strAddressSegment == NnlType::STR_NULL )
    {
      return false;
    }

    const int nAddressValue = atoi( strAddressSegment );

    if ( nAddressValue < 0 || nAddressValue > 255 )
    {
      return false;
    }

    //2 Get the next segment
    strAddressSegment = _tcstok( NULL, pcAddressSeparator );
  }

  strAddress.ReleaseBuffer( -1 );
  return true;
}

///////////////////////////////////////////////////////////////////////////////
unsigned long CnlNetworkModel::MakeMsbMask(
  unsigned int nLength ) const
{
  //1 Error checking
  if ( nLength > nNumBitsInIPv4Address )
  {
    return 0;
  }

  if ( nLength == nNumBitsInIPv4Address )
  {
    return 0xffffffff;
  }

  const unsigned int nMask = ( 1 << nLength ) - 1;
  const unsigned int nAlignedMask = nMask << ( 32 - nLength );
  return nAlignedMask;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::SetConsoleMessage(
  const CnlCompositeHandle hSourceMachine,
  const CString strMessage )
{
  // For now, handle console messages as ponit of focus messages
  CString strConsoleMessage = _T( "Console: " ) + strMessage;
  return SuspendAtPof( 
    hSourceMachine,
    LAYER_UPPER,
    EVENT_TYPE_INFO,
    strConsoleMessage );
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::GetProcessingMachineDescr(
  const CnlCompositeHandle hProcessingMachine ) const
{
  CString strId;
  strId.Format( _T( "%d" ), hProcessingMachine.ID() );
  return strId;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::GetProcessingMachineIpAddress(
  CString& strIpAddress,
  const CnlCompositeHandle hProcessingMachine,
  const unsigned int nNicOffset ) const
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;
      eResult = Imp()->GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 No such network
        return eResult;
      }

      return pNetwork->GetProcessingMachineIpAddress(
        strIpAddress, 
        hProcessingMachine,
        nNicOffset );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->GetIpAddress( strIpAddress, nNicOffset );
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::GetProcessingMachineMacAddress(
  CString& strMacAddress,
  const CnlCompositeHandle hProcessingMachine,
  const unsigned int nNicOffset ) const
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;
      eResult = Imp()->GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 No such network
        return eResult;
      }

      return pNetwork->GetProcessingMachineMacAddress(
        strMacAddress, 
        hProcessingMachine,
        nNicOffset );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->GetMacAddress( strMacAddress, nNicOffset );
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::GetProcessingMachineNicCount(
  unsigned int& nNumNics,
  const CnlCompositeHandle hProcessingMachine ) const
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;
      eResult = Imp()->GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 No such network
        return eResult;
      }

      return pNetwork->GetProcessingMachineNicCount(
        nNumNics, 
        hProcessingMachine );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->GetNicCount( nNumNics );
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}


///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::GetProcessingMachineString(
  CString& strConsoleOutput,
  const CnlCompositeHandle hProcessingMachine,
  NnlSystemMgrType::EnlCommandLineEntry eEntry ) const
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;
      eResult = Imp()->GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 No such network
        return eResult;
      }

      return pNetwork->GetProcessingMachineString(
        strConsoleOutput, 
        hProcessingMachine,
        eEntry );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->GetString(
        strConsoleOutput, 
        eEntry );
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::ToggleProcessingMachinePower(
  const CnlCompositeHandle hProcessingMachine )
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;
      eResult = Imp()->GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 No such network
        return eResult;
      }

      return pNetwork->ToggleProcessingMachinePower(
        hProcessingMachine );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->TogglePower();
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}


///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::GetProcessingMachinePowerStatus(
  bool& bIsOn,
  const CnlCompositeHandle hProcessingMachine ) const
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;
      eResult = Imp()->GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 No such network
        return eResult;
      }

      return pNetwork->GetProcessingMachinePowerStatus(
        bIsOn, 
        hProcessingMachine );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->GetPowerStatus(
        bIsOn );
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // We don't currently allow this
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}

///////////////////////////////////////////////////////////////////////////////
NnlSystemMgrType::EnlResult CnlNetworkModel::ClearProcessingMachineArpCache(
  const CnlCompositeHandle hProcessingMachine)
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      //2 End machines are currently embedded in networks
      CnlNetwork* pNetwork = NULL;
      eResult = Imp()->GetNetwork( pNetwork, hProcessingMachine.Composite() );

      if ( eResult != RESULT_OK )
      {
        //3 No such network
        return eResult;
      }

      return pNetwork->ClearProcessingMachineArpCache(
        hProcessingMachine );
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->ClearArpCache();
    }
    case HANDLE_TYPE_NETWORK :
    {
      assert( 0 );  // Nonsense
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}

///////////////////////////////////////////////////////////////////////////////
bool CnlNetworkModel::IsCommandExecuting() const
{
  return ( Imp()->m_eThreadStatus != THREAD_STATUS_COMPLETED );
}

///////////////////////////////////////////////////////////////////////////////
NnlSystemMgrType::EnlResult CnlNetworkModel::SetShowEventType(
  NnlSystemMgrType::EnlEventType eEventType,
  const bool bShow )
{
  switch( eEventType )
  {
    case EVENT_TYPE_SEND :
    case EVENT_TYPE_RECEIVE_ACCEPT:
    {
      Imp()->m_ShowEventSettings[ eEventType ] = bShow;
      NotifyObservers( MESSAGE_SETTINGS_CHANGED );
      return RESULT_OK;
    }
    case EVENT_TYPE_UNKNOWN :
    case NUM_EVENT_TYPES :
    case EVENT_TYPE_INFO :
    case EVENT_TYPE_RECEIVE_REJECT :
    {
      return RESULT_ERROR_UNSUPPORTED_OPERATION;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_UNSUPPORTED_OPERATION;
}
  
///////////////////////////////////////////////////////////////////////////////
bool CnlNetworkModel::GetShowEventType(
  NnlSystemMgrType::EnlEventType eEventType ) const
{
  return Imp()->m_ShowEventSettings [ eEventType ];
}

///////////////////////////////////////////////////////////////////////////////
void CnlNetworkModel::EnablePofProcessing( 
  const bool bEnable )
{
  Imp()->m_bPofEnabled = bEnable;
}

///////////////////////////////////////////////////////////////////////////////
void CnlNetworkModel::SetLayerFilter(
  const NnlSystemMgrType::EnlLayer eLowestLayerToShow )
{
  Imp()->m_eLowestLayerToShow = eLowestLayerToShow;
  NotifyObservers( MESSAGE_SETTINGS_CHANGED );
}

///////////////////////////////////////////////////////////////////////////////
EnlLayer CnlNetworkModel::GetLayerFilter() const
{
  return Imp()->m_eLowestLayerToShow;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::GetLayerString(
  const NnlSystemMgrType::EnlLayer eLayer ) const
{
  return Imp()->m_LayerStrings[ eLayer ];
}

///////////////////////////////////////////////////////////////////////////////
NnlSystemMgrType::EnlResult CnlNetworkModel::AddNicToRouter(
  const CnlCompositeHandle hProcessingMachine,
  NnlSystemMgrType::EnlRouterNicType eNicType )
{
  //1 Error checking
  EnlResult eResult = RESULT_UNKNOWN;

  switch( hProcessingMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      return RESULT_ERROR_WRONG_MACHINE_TYPE;
    }
    case HANDLE_TYPE_ROUTER :
    {
      //2 Model currenlty only supports 1 router
      return Imp()->m_pRouter->AddNic( eNicType );
    }
    case HANDLE_TYPE_NETWORK :
    {
      return RESULT_ERROR_INVALID_HANDLE;
    }
    default : assert( 0 );  // Don't know this type
  }

  return RESULT_ERROR_INVALID_HANDLE;  
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::TransportProtocolToString(
  NnlSystemMgrType::EnlTransportProtocol eProtocol ) const
{
  switch( eProtocol )
  {
    case TRANSPORT_PROTOCOL_UNKNOWN :
    {
      return _T( "UNKNOWN" );
    }
    case TRANSPORT_PROTOCOL_NONE :
    {
      return _T( "NONE" );
    }
    case TRANSPORT_PROTOCOL_ROUTER_QUERY :
    {
      return _T( "ROUTER QUERY" );
    }
    case TRANSPORT_PROTOCOL_ROUTER_RESPONSE :
    {
      return _T( "ROUTER RESPONSE" );
    }
    case TRANSPORT_PROTOCOL_PING :
    {
      return _T( "PING" );
    }
    case TRANSPORT_PROTOCOL_RPING :
    {
      return _T( "RPING" );
    }
    case TRANSPORT_PROTOCOL_TCP :
    {
      return _T( "TCP" );
    }
    case TRANSPORT_PROTOCOL_UDP :
    {
      return _T( "UDP" );
    }
    default : assert( 0 );  // Don't know this protocol
  }

  return STR_NULL;
}
///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::NetworkProtocolToString(
  NnlSystemMgrType::EnlNetworkProtocol eProtocol ) const
{
  switch( eProtocol )
  {
    case NETWORK_PROTOCOL_UNKNOWN :
    {
      return _T( "UNKNOWN" );
    }
    case NETWORK_PROTOCOL_ARP:
    {
      return _T( "ARP" );
    }
    case NETWORK_PROTOCOL_IP:
    {
      return _T( "IP" );
    }
    case NETWORK_PROTOCOL_RARP :
    {
      return _T( "RARP" );
    }
    default : assert( 0 );  // Don't know this protocol
  }

  return STR_NULL;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::LinkProtocolToString(
  NnlSystemMgrType::EnlLinkProtocol eProtocol ) const
{
  switch( eProtocol )
  {
    case LINK_PROTOCOL_UNKNOWN :
    {
      return _T( "UNKNOWN" );
    }
    case LINK_PROTOCOL_ETHERNET :
    {
      return _T( "ETHERNET" );
    }
    default : assert( 0 );  // Don't know this protocol
  }

  return STR_NULL;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNetworkModel::ResultToString(
  NnlSystemMgrType::EnlResult eResult ) const
{
  switch( eResult )
  {
    case RESULT_OK :
    {
      return _T( "OK" );
    }
    case RESULT_EXECUTION_INCOMPLETE :
    {
      return _T( "Execution incomplete" );
    }
    case RESULT_LAYER_FILTERED_OUT :
    {
      return _T( "Specified layer filtered out" );
    }
    case RESULT_EVENT_TYPE_FILTERED_OUT :
    {
      return _T( "Event type filtered out" );
    }
    case RESULT_ERROR_ALREADY_CONNECTED :
    {
      return _T( "Error: Already connected" );
    }
    case RESULT_ERROR_DUPLICATE_ADDRESS :
    {
      return _T( "Error: Duplicate address" );
    }
    case RESULT_ERROR_END_MACHINE_DOES_NOT_EXIST :
    {
      return _T( "Error: End Machine does not exist" );
    }
    case RESULT_ERROR_INVALID_ADDRESS :
    {
      return _T( "Error: Invalid address" );
    }
    case RESULT_ERROR_INVALID_ADDRESS_LENGTH :
    {
      return _T( "Error: Invalid address length" );
    }
    case RESULT_ERROR_INVALID_HANDLE :
    {
      return _T( "Error: Invalid handle" );
    }
    case RESULT_ERROR_MACHINE_IS_POWERED_OFF :
    {
      return _T( "Error: Machine is powered fff" );
    }
    case RESULT_ERROR_MAX_EXCEEDED :
    {
      return _T( "Error: Max exceeded" );
    }
    case RESULT_ERROR_NETWORK_DOES_NOT_EXIST :
    {
      return _T( "Error: Network does not exist" );
    }
    case RESULT_ERROR_NIC_DOES_NOT_EXIST :
    {
      return _T( "Error: NIC does not exist" );
    }
    case RESULT_ERROR_NO_COMMAND_EXECUTING :
    {
      return _T( "Error: No command executing" );
    }
    case RESULT_ERROR_PREVIOUS_EXECUTION_IMCOMPLETE :
    {
      return _T( "Error: Previous execution incomplete" );
    }
    case RESULT_ERROR_UNKNOWN_FRAME_TYPE:
    {
      return _T( "Error: Unknown frame type" );
    }
    case RESULT_ERROR_UNSUPPORTED_OPERATION:
    {
      return _T( "Error: Unsupported operation" );
    }
    case RESULT_ERROR_WRONG_HANDLE_TYPE:
    {
      return _T( "Error: Wrong handle type" );
    }
    case RESULT_ERROR_WRONG_MACHINE_TYPE :
    {
      return _T( "Error: Wrong machine type" );
    }
    case RESULT_ERROR_WRONG_NETWORK :
    {
      return _T( "Error: Wrong network" );
    }
    default : assert( 0 );  // Don't know this protocol
  }

  return STR_NULL;
}

