//////////////////////////////////////////////////////////////////////////////
//
// @descr    Primary model class
//
// @short    Primary model class
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_NETWORK_MODEL_H__
#define __NL_NETWORK_MODEL_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

// From Utilities
#include <nlModel.h>
#include <nlObserver.h>

// Local
#include "nlClientInterface.h"
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlNetworkModel : public CnlModel, public CnlClientInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlNetworkModel();

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    // @descr   Unit test
    //          Preconditions: None
    //          Postconditions(dependencies):
    //            a. (none) Runs a complete unit test of this class
    // @return  True if passed; False otherwise
    // @end
    bool UnitTest();

    // @descr   Stop command processing at a point of focus, enabling the user to 
    //          view activity.
    //          Preconditions:
    //            1. The specified protocol layer is at or above the filter level
    //          Postconditions(dependencies):
    //            a. (none) End Machine marked, all others unmarked; message set
    //                    and the command thread suspended
    // @param   hProcessingMachine  IN: Handle to the end machine to mark
    // @param   eLayer  IN: Protocol layer of initiator
    // @param   bIsRejection  IN: True if this is a rejection-type message
    // @param   strMessage  IN: Message to associate with the hot spot
    // @return  Request result
    // @end
    NnlSystemMgrType::EnlResult SuspendAtPof(
      const CnlCompositeHandle hPof,
      const NnlSystemMgrType::EnlLayer eLayer,
      const NnlSystemMgrType::EnlEventType eEventType,
      const CString& strMessage );

    // @descr   Generate a unique MAC address for a Nic
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Address returned
    // @return  MAC address
    // @end
    CString MakeMacAddress();

    // @descr   Convert an IPv4 dot notation string to a numeric value
    //          Preconditions:
    //            1. Valid format
    //          Postconditions(dependencies):
    //            a. (all) Value returned; else -1 returned
    // @param   nAddress  OUT: Equivalent numeric value
    // @param   strDotNotation  IN: Dot notation string
    // @param   eIpVersion  IN: Version of IP to use
    // @return  Reqest result
    // @end
    NnlSystemMgrType::EnlResult DotToNumeric(
      unsigned long& nAddress,
      const CString& strDotNotation,
      NnlSystemMgrType::EnlIpVersion eIpVersion ) const;

    // @descr   Convert a numeric value to an IPv4 dot notation string
    //          Preconditions:
    //            1. Valid value
    //          Postconditions(dependencies):
    //            a. (all) String returned; else NULL returned
    // @param   strDotNotation    OUT: Equivalent dot notation string
    // @param   nAddress  IN: Numeric value
    // @param   eIpVersion  IN: Version of IP to use
    // @return  Reqest result
    // @end
    NnlSystemMgrType::EnlResult NumericToDot(
      CString& strDotNotation,
      const unsigned long nAddress,
      NnlSystemMgrType::EnlIpVersion eIpVersion ) const;   
    
    // @descr   Indicates if the dot notation string is valid
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Result returned
    // @param   strDotNotation  IN: String to check
    // @param   eIpVersion  IN: IP version to use
    // @return  True if valid; False otherwise
    // @end
    bool IsValidDotNotation(
      const CString& strDotNotation,
      NnlSystemMgrType::EnlIpVersion eIpVersion ) const;

    // @descr   Make an msb mask of the specified length
    //          Preconditions:
    //            1. IPv4-compatible length
    //          Postconditions(dependencies):
    //            a. (all) Valid mask created; else all 0s
    // @param   nLength  IN: Length of mask to create.
    // @return  Mask
    // @end
    unsigned long MakeMsbMask(
      unsigned int nLength ) const;


    // @descr   Save a "console" message. Observers will be notified so they
    //          can display the message.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Message saved.
    // @param   hSourceMachine  IN: Machine generating the message
    // @param   strMessage  IN: Message to save
    // @return  Request result
    // @end
    NnlSystemMgrType::EnlResult SetConsoleMessage(
      const CnlCompositeHandle hSourceMachine,
      const CString strMessage );

    CString TransportProtocolToString(
      NnlSystemMgrType::EnlTransportProtocol eProtocol ) const;
    CString NetworkProtocolToString(
      NnlSystemMgrType::EnlNetworkProtocol eProtocol ) const;
    CString LinkProtocolToString(
      NnlSystemMgrType::EnlLinkProtocol eProtocol ) const;
    
    //2 From CnlClientInterface
    NnlSystemMgrType::EnlResult AttachObserver(
      CnlObserver* const );
    NnlSystemMgrType::EnlResult DetachObserver(
      CnlObserver* const );
    void NotifyObserversOfChange(
      NnlSystemMgrType::EnlMessage eMessage );
    CString ResultToString(
      NnlSystemMgrType::EnlResult eResult ) const;
    NnlSystemMgrType::EnlResult AddNetworkToInternet(
      CnlCompositeHandle&,
      const CString& );
    NnlSystemMgrType::EnlResult AddProcessingMachineToNetwork(
      CnlCompositeHandle&,
      const CnlCompositeHandle );
    NnlSystemMgrType::EnlResult MakeRouter(
      CnlCompositeHandle& );    
    NnlSystemMgrType::EnlResult AssignIpAddress(
      const CnlCompositeHandle,
      const unsigned int,
      const CString& );
    NnlSystemMgrType::EnlResult ConnectRouter(
      const CnlCompositeHandle,
      const unsigned int );
    const CnlCommand* MakeCommandArp(
      const CString& ) const;
    const CnlCommand* MakeCommandPing(
      const CString& ) const;
    NnlSystemMgrType::EnlResult ExecuteCommand(
      const CnlCompositeHandle,
      const CnlCommand* const );
    CnlCompositeHandle GetPofHandle();
    CString GetPofMessage();
    unsigned int GetPofSequenceNum() const;
    NnlSystemMgrType::EnlResult SingleStep();
    NnlSystemMgrType::EnlResult RunToEnd();
    void Test();
    CString GetProcessingMachineDescr(
      const CnlCompositeHandle ) const;
    NnlSystemMgrType::EnlResult GetProcessingMachineIpAddress(
      CString& strIpAddress,
      const CnlCompositeHandle hProcessingMachine,
      const unsigned int nNicOffset ) const;
    NnlSystemMgrType::EnlResult GetProcessingMachineMacAddress(
      CString& strMacAddress,
      const CnlCompositeHandle hProcessingMachine,
      const unsigned int nNicOffset ) const;
    NnlSystemMgrType::EnlResult GetProcessingMachineNicCount(
      unsigned int&,
      const CnlCompositeHandle ) const;
    NnlSystemMgrType::EnlResult GetProcessingMachineString(
      CString&,
      const CnlCompositeHandle,
      NnlSystemMgrType::EnlCommandLineEntry ) const;
    NnlSystemMgrType::EnlResult ToggleProcessingMachinePower(
      const CnlCompositeHandle );
    NnlSystemMgrType::EnlResult GetProcessingMachinePowerStatus(
      bool&,
      const CnlCompositeHandle ) const;
    void SetLayerFilter(
      const NnlSystemMgrType::EnlLayer );
    NnlSystemMgrType::EnlLayer GetLayerFilter() const;
    CString GetLayerString(
      const NnlSystemMgrType::EnlLayer ) const;
    NnlSystemMgrType::EnlResult AddNicToRouter(
      const CnlCompositeHandle,
      NnlSystemMgrType::EnlRouterNicType );
    NnlSystemMgrType::EnlResult ClearProcessingMachineArpCache(
      const CnlCompositeHandle);
    bool IsCommandExecuting() const;
    NnlSystemMgrType::EnlResult SetShowEventType(
      NnlSystemMgrType::EnlEventType,
      const bool );
    bool GetShowEventType(
      NnlSystemMgrType::EnlEventType ) const;
    void EnablePofProcessing( 
      const bool );

    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlNetworkModel();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    friend class CnlExecuteCommandThread;
    friend class CnlExecuteCommandThreadHandler;

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlNetworkModel( const CnlNetworkModel& );
    CnlNetworkModel& operator=( const CnlNetworkModel& );
    CnlNetworkModel* operator&();
    const CnlNetworkModel* operator&() const;
};

#endif

