//////////////////////////////////////////////////////////////////////////////
//
// @descr    SystemMgr Implementation Header
//
// @short    SystemMgr Implementation Header
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_SYSTEM_MGR_IMP_H__
#define __NL_SYSTEM_MGR_IMP_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From Utilities
#include <nlCompositeHandle.h>

// Local
#include "nlNetworkModel.h"
#include "nlSystemMgrType.h"
#include "nlNetwork.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlIntegerPool;
class CnlIPv4Address;
class CnlProcessingMachine;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
struct CnlNetworkModel::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlNetworkModel* const pOwner );

  // Destructor
   ~SnlImp();


  bool IsNumProcessingMachinesValidForSubnet( 
    const int nSubnetAddressLength,
    const int nNumProcessingMachines );

  NnlSystemMgrType::EnlResult GetNetwork(
    CnlNetwork*& pNetwork,
    const CnlCompositeHandle hNetwork ) const;

  NnlSystemMgrType::EnlResult MakeAndRegisterNetwork(
    CnlCompositeHandle&,
    const CnlIPv4Address& address );

  void DestroyNetwork(
    CnlCompositeHandle );

  // Version called by the worker thread
  NnlSystemMgrType::EnlResult ExecuteCommand(
    const CnlCompositeHandle,
    const CnlCommand* const );

  // Follow up on the results of the command
  void PostExecuteCommand(
    const CnlCompositeHandle,
    const CnlCommand* const );
  
  // Generate a random lowercase letter
  CString MakeRandomChar();

  // @descr   Ensure the address is globally unique.
  //          Preconditions:
  //            1. None 
  //          Postconditions(dependencies):
  //            a. (none) Result returned
  // @param   strAddress  IN: Address to check
  // @return  True if unique; False otherwise
  // @end
  bool IsAddressUnique( 
    const CString& strAddress ) const;

  //          nMinNumHostBits and nMaxNumHostBits
  //          Preconditions:
  //            1. None 
  //          Postconditions(dependencies):
  //            a. (none) Result returned
  // @param   nLength  IN: Length to check
  // @return  True if valid; False otherwise
  // @end
  bool ValidateAddressLength(
    const int nLength ) const;

  //2 Unit Test
  //bool UnitTest();

  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CnlNetworkModel* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')
  // The set of all MAC addresses ever created
  std::set< CString >m_MacAddressRegistry;

  // The set of all network addresses known in this internet
  // This does not include end machines or routers!
  std::set< CString >m_NetworkAddressRegistry;

  // A mapping between network handle IDs and the networks they are assigned to
  std::map< unsigned long, CnlNetwork* > m_NetworkHandleMap;

  // My one and only router, if any
  CnlProcessingMachine* m_pRouter;
  
  // A pool of handle IDs
  CnlIntegerPool* const m_pHandleIdPool;

  // The end machine that is currntly the point of focus
  CnlCompositeHandle m_hPof;
  
  // The message to display regarding the point of focus
  CString m_strPofMessage;

  // The point of focus message sequence number
  unsigned int m_nPofSequenceNum;
  
  // The console message to display
  CString m_strConsoleMessage;
  
  // Handle to the thread I'm currently running in
  HANDLE m_hThisThread;

  // The return value from the threaded ExecuteCommand call
  NnlSystemMgrType::EnlResult m_eResultFromThread;

  // The thread which handles command execution
  CnlExecuteCommandThread* m_pExecuteCommandThread;

  NnlSystemMgrType::EnlThreadStatus m_eThreadStatus;
  
  // The message the thread said needed to be sent to observers
  NnlSystemMgrType::EnlMessage m_eMessageFromThread;

  // The current activity display filter level
  NnlSystemMgrType::EnlLayer m_eLowestLayerToShow;

  // Sting descriptions of each layer
  std::vector< CString > m_LayerStrings;

  // Should I display pof messages of the specified event types?
  std::vector< bool > m_ShowEventSettings;

  // Is Point of Focus processing enabled?
  bool m_bPofEnabled;
  
private:

  // Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

#endif

