//////////////////////////////////////////////////////////////////////////////
//
// @descr    The operating system running in an end machine
//
// @short    The operating system running in an end machine
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_Os_H__
#define __NL_Os_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgrType.h"
#include "nlProcessInterface.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlHwSwInterface;
class CnlVariableParmContainer;
class CnlCompositeHandle;
class CnlFrame;
class CnlIPv4Address;
class CnlRoutingTable;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlOs : public CnlProcessInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @param   hMachine  IN: User handle to the machine
    // @param   pHardware  IN: Connection to this machine's hardware
    // @end
    explicit CnlOs(
      const CnlCompositeHandle hMachine,
      CnlHwSwInterface* const pHardware );

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    // @descr   Get the interface to the hardare
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Hardware interface returned;
    // @return  Hardware interface
    // @end
    CnlHwSwInterface* GetHardwareInterface() const;

    // @descr   See CnlNetworkModel::SetConsoleOutput
    // @end
    void SetConsoleMessage(
      const CString strMessage );

    // @descr   Return the contents of an arp -a query as a string
    // @end
    CString GetArpCacheString() const;

    // @descr   Return the contents of a netstat -rn query as a string
    // @end
    CString GetNetstatString() const;

    // @descr   Process a frame coming in from the network
    //          Preconditions:
    //            1. Frame type is supported
    //          Postconditions(dependencies):
    //            a. (none) Frame processed
    // @param   frame  IN: Frame to process
    // @return  Request result
    // @end
    NnlSystemMgrType::EnlResult ReceiveFrame(
      const CnlFrame& frame );

    //2 From CnlProcessInterface
    void ReceiveEndPayload(
      const CnlVariableParmContainer& );

    //2 Forwarded by CnlProcessingMachine
    NnlSystemMgrType::EnlResult AssignIpAddress(
      const unsigned int,
      const CString& );
    NnlSystemMgrType::EnlResult GetIpAddress(
      CString& strIpAddress,
      const unsigned int nNicOffset ) const;

    //2 See CnlIpModule
    void SendArp(
      const CString& strDestinationIp );
    void SendPing(
      const CString& strDestinationIp ); 
    void VerifyArpResult( 
      const CString& strDestinationIp );
    void VerifyPingResult( 
      const CString& strDestinationIp );
    void ClearArpCache();

    void Boot();
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlOs();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlOs( const CnlOs& );
    CnlOs& operator=( const CnlOs& );
    CnlOs* operator&();
    const CnlOs* operator&() const;
};

#endif

