/////////////////////////////////////////////////////////////////////////////
//
// @descr    A host in a subnet
//
// @short    A host in a subnet
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilties
#include <nlCommonType.h>
#include <nlCompositeHandle.h>

//1 Local
#include "nlProcessingMachine.h"
#include "nlNic.h"
#include "nlSystemMgr.h"
#include "nlNetworkModel.h"
#include "nlOs.h"
#include "nlCommand.h"
#include "nlIPv4Address.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::vector< CnlNic* >::const_iterator NicIter;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlProcessingMachine::SnlImp* CnlProcessingMachine::Imp()       { return m_pImp; }
inline const  CnlProcessingMachine::SnlImp* CnlProcessingMachine::Imp() const { return m_pImp; }

struct CnlProcessingMachine::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlProcessingMachine* const pOwner,
    CnlCompositeHandle hThis,
    const NnlSystemMgrType::EnlMachineType eType );

  // Destructor
   ~SnlImp();

  void Boot();
  
  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CnlProcessingMachine* const m_pOwner;

  // Handle used externally
  const CnlCompositeHandle m_hThis;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // My type
  const NnlSystemMgrType::EnlMachineType m_eType;

  // My network cards
  //3 Policy: End Machines get 1 NIC; routers get unlimited number
  std::vector< CnlNic* > m_Nics;

  // My operating system
  CnlOs* const m_pOs;

  // My power status
  bool m_bIsOn;

  // My console text
//  std::queue< CString> m_ConsoleText;
  
private:

  // Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlProcessingMachine::SnlImp::SnlImp(
  CnlProcessingMachine* const pOwner,
  CnlCompositeHandle hThis,
  const NnlSystemMgrType::EnlMachineType eType )
 :
  m_pOwner( pOwner ),
  m_hThis( hThis ),
  m_eType( eType ),
  m_pOs( new CnlOs( m_hThis, m_pOwner ) ),
  m_bIsOn( true )
{
  switch( eType )
  {
    case MACHINE_TYPE_END_MACHINE :
    {
      m_Nics.push_back( new CnlNic( m_pOwner ) );
      break;
    }
    case MACHINE_TYPE_ROUTER :
    {
      //3 User adds these manually
      break;
    }
    default : assert( 0 );  // Don't kow this type
  }

}

///////////////////////////////////////////////////////////////////////////////
CnlProcessingMachine::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlProcessingMachine::SnlImp::Boot()
{
  m_pOs->Boot();
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlProcessingMachine::CnlProcessingMachine(
  CnlCompositeHandle hThis,
  const NnlSystemMgrType::EnlMachineType eType )
{
  m_pImp = new SnlImp( this, hThis, eType  );
}

///////////////////////////////////////////////////////////////////////////////
CnlProcessingMachine::~CnlProcessingMachine()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlProcessingMachine::Initialize()
{
  Imp()->m_pOs->Initialize();
}

///////////////////////////////////////////////////////////////////////////////
void CnlProcessingMachine::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
bool CnlProcessingMachine::IsMachineOn() const
{
  return Imp()->m_bIsOn;
}

///////////////////////////////////////////////////////////////////////////////
CnlNic* CnlProcessingMachine::GetNic(
  const unsigned int nNicId ) const
{
  return Imp()->m_Nics[ nNicId ];
}

///////////////////////////////////////////////////////////////////////////////
CnlOs* CnlProcessingMachine::GetOs() const
{
  return Imp()->m_pOs;
}

 ///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::AssignIpAddress(
  const unsigned int nNicId,
  const CString& strClasslessIpAddress )
{
  if ( ! Imp()->m_bIsOn )
  {
    return RESULT_ERROR_MACHINE_IS_POWERED_OFF;
  }
  
  return Imp()->m_pOs->AssignIpAddress( nNicId, strClasslessIpAddress );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::ExecuteCommand(
  const CnlCommand* const pCommand )
{
  if ( ! Imp()->m_bIsOn )
  {
    return RESULT_ERROR_MACHINE_IS_POWERED_OFF;
  }
  
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();

  CString strMessage;

  switch( pCommand->GetType() )
  {
    case COMMAND_ARP :
    {
      Imp()->m_pOs->SendArp( pCommand->GetParms().GetStringParameter( 0 ) );
      break;
    }
    case COMMAND_PING :
    {
      Imp()->m_pOs->SendPing( pCommand->GetParms().GetStringParameter( 0 ) );
      break;
    }

    default :
    {
      assert( 0 );  // Don't know this command
    }
  }

  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::PostExecuteCommand(
  const CnlCommand* const pCommand )
{
  if ( ! Imp()->m_bIsOn )
  {
    return RESULT_ERROR_MACHINE_IS_POWERED_OFF;
  }
  
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();

  CString strMessage;

  switch( pCommand->GetType() )
  {
    case COMMAND_ARP :
    {
      Imp()->m_pOs->VerifyArpResult(  
        pCommand->GetParms().GetStringParameter( 0 ) );
      break;
    }
    case COMMAND_PING :
    {
      Imp()->m_pOs->VerifyPingResult(  
        pCommand->GetParms().GetStringParameter( 0 ) );
      break;
    }

    default :
    {
      assert( 0 );  // Don't know this command
    }
  }

  return RESULT_OK;
}


///////////////////////////////////////////////////////////////////////////////
CnlNic* CnlProcessingMachine::GetNic(
  const unsigned int nNicId )
{
  //1 Error checking
  if ( nNicId >= Imp()->m_Nics.size() )
  {
    return NULL;
  }

  return Imp()->m_Nics[ nNicId ];
}

///////////////////////////////////////////////////////////////////////////////
CnlCompositeHandle CnlProcessingMachine::GetProcessingMachineHandle() const
{
  return Imp()->m_hThis;
}

///////////////////////////////////////////////////////////////////////////////
unsigned int CnlProcessingMachine::GetNumNics() const
{
  if ( ! Imp()->m_bIsOn )
  {
    return 0;
  }
  
  return ( Imp()->m_Nics.size() );
}

///////////////////////////////////////////////////////////////////////////////
NnlSystemMgrType::EnlResult CnlProcessingMachine::AddNic(
  NnlSystemMgrType::EnlRouterNicType eNicType )
{
  //1 Error checking
  if ( Imp()->m_eType != MACHINE_TYPE_ROUTER )
  {
    return RESULT_ERROR_WRONG_MACHINE_TYPE;
  }
  
  switch ( eNicType )
  {
    case ROUTER_NIC_TYPE_NETWORK :
    {
      Imp()->m_Nics.push_back( new CnlNic( this ) );
      return RESULT_OK;
      break;
    }
    case ROUTER_NIC_TYPE_GATEWAY :
    {
      //3 Not currently supported
      break;
    }
    default : assert( 0 );  // Don't know ths type
  }

  return RESULT_ERROR_UNSUPPORTED_OPERATION;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::GetIpAddress(
  CString& strIpAddress,
  const unsigned int nNicOffset ) const
{
  if ( ! Imp()->m_bIsOn )
  {
    return RESULT_ERROR_MACHINE_IS_POWERED_OFF;
  }
  
  return Imp()->m_pOs->GetIpAddress( strIpAddress, nNicOffset );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::GetMacAddress(
  CString& strMacAddress,
  const unsigned int nNicOffset ) const
{
  if ( ! Imp()->m_bIsOn )
  {
    return RESULT_ERROR_MACHINE_IS_POWERED_OFF;
  }
  
  if ( nNicOffset >= Imp()->m_Nics.size() )
  {
    return RESULT_ERROR_NIC_DOES_NOT_EXIST;
  }
  
  strMacAddress = Imp()->m_Nics[ nNicOffset ]->GetMacAddress();

  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::GetNicCount(
  unsigned int& nCount ) const
{
  if ( ! Imp()->m_bIsOn )
  {
    return RESULT_ERROR_MACHINE_IS_POWERED_OFF;
  }
  
  nCount = Imp()->m_Nics.size();
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::GetString(
  CString& strConsoleOutput,
  NnlSystemMgrType::EnlCommandLineEntry eEntry ) const
{
  if ( ! Imp()->m_bIsOn )
  {
    return RESULT_ERROR_MACHINE_IS_POWERED_OFF;
  }
  
  switch( eEntry )
  {
    case COMMAND_LINE_ENTRY_ARP_A :
    {
      strConsoleOutput = Imp()->m_pOs->GetArpCacheString();
      break;
    }
    case COMMAND_LINE_ENTRY_NETSTAT_RN :
    {
      strConsoleOutput = Imp()->m_pOs->GetNetstatString();
      break;
    }

    default :
    {
      assert( 0 );  // Don't know this type
      break;
    }
  }

  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::TogglePower()
{
  Imp()->m_bIsOn = ! Imp()->m_bIsOn;

  if ( Imp()->m_bIsOn )
  {
    Imp()->Boot();
  }
  
  CnlSystemMgr::Instance().GetInternalInterface()->NotifyObserversOfChange(
    MESSAGE_POWER_STATUS_CHANGED );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::GetPowerStatus(
  bool& bIsOn ) const
{
  bIsOn = Imp()->m_bIsOn;
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlProcessingMachine::ClearArpCache()
{
  if ( ! Imp()->m_bIsOn )
  {
    return RESULT_ERROR_MACHINE_IS_POWERED_OFF;
  }
  
  Imp()->m_pOs->ClearArpCache();
  return RESULT_OK;
}

