/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlRoutingTable.h
//
// @short    See nlRoutingTable.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>

//1 Local
#include "nlRoutingTable.h"
#include "nlIPv4Address.h"
#include "nlRoutingTableRow.h"
#include "nlSystemMgrType.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::vector< CnlRoutingTableRow* >::const_iterator ConstRowIter;
  typedef std::map< unsigned int, CnlIPv4Address >::const_iterator ConstNicIpMapIter;
  typedef std::map< CnlIPv4Address, unsigned int >::const_iterator ConstIpNicMapIter;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlRoutingTable::SnlImp* CnlRoutingTable::Imp()       { return m_pImp; }
inline const  CnlRoutingTable::SnlImp* CnlRoutingTable::Imp() const { return m_pImp; }

struct CnlRoutingTable::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlRoutingTable* const pOwner );

  // Destructor
   ~SnlImp();

  // Select the row that is the best match for the specified IP destination
  const CnlRoutingTableRow* CnlRoutingTable::SnlImp::SelectBestMatchRow( 
    const CString& strIpToLookup ) const;

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlRoutingTable* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The collection of rows
  std::vector< CnlRoutingTableRow* > m_Table;

  // The mapping between IP addresses and Nic cards
  std::map< CnlIPv4Address, unsigned int > m_IpNicMap;
  std::map< unsigned int, CnlIPv4Address > m_NicIpMap;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTable::SnlImp::SnlImp(
  CnlRoutingTable* const pOwner )
 :
  m_pOwner( pOwner )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTable::SnlImp::~SnlImp()
{
  for ( ConstRowIter iter = m_Table.begin(); iter != m_Table.end(); ++iter )
  {
    delete ( *iter );
  }
}

///////////////////////////////////////////////////////////////////////////////
const CnlRoutingTableRow* CnlRoutingTable::SnlImp::SelectBestMatchRow( 
  const CString& strIpToLookup ) const
{
  std::stack< const CnlRoutingTableRow* > rowMatches;
  const CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  unsigned long nIp = 0;
  const EnlResult eResult = pNetworkModel->DotToNumeric( 
    nIp, 
    strIpToLookup, 
    IP_VERSION_4 );
    
  for ( ConstRowIter iter = m_Table.begin(); iter != m_Table.end(); ++iter )
  {
    const unsigned long nDestination = ( *iter )->GetDestination();
    const unsigned long nNetMask = ( *iter )->GetNetmask();

    if ( ( nIp & nNetMask ) == nDestination )
    {
      rowMatches.push( *iter );
    }
  }

  if ( rowMatches.size() > 0 )
  {
    return rowMatches.top();
  }
  else
  {
    return NULL;
  }
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlRoutingTable::CnlRoutingTable()
{
  m_pImp = new SnlImp( this );
}

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTable::~CnlRoutingTable()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlRoutingTable::AddRow(
  const CString strDestination,
  const CString strMask,
  const CString strGateway,
  const CString strInterface )
{
  Imp()->m_Table.push_back( 
    new CnlRoutingTableRow(
      strDestination,
      strMask,
      strGateway,
      strInterface ) );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlRoutingTable::AssignIpAddress(
  const unsigned int nNicId,
  const CString& strClasslessIpAddress )
{
  CnlIPv4Address address( strClasslessIpAddress );
  
  if ( address.Validity() != RESULT_OK )
  {
    return address.Validity();
  }
  
  Imp()->m_IpNicMap[ address ] = nNicId;
  Imp()->m_NicIpMap[ nNicId ] = address;

  //2 Notify observers
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  pNetworkModel->NotifyObserversOfChange( MESSAGE_IP_ADDRESS_CHANGED );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlRoutingTable::AsString() const
{
  CString strTitle;
    strTitle.Format(
      strNetstatFormat,
      _T( "Destination" ),
      _T( "Netmask" ),
      _T( "Gateway" ),
      _T( "Interface" ) );

  CString strContents = strTitle;

  for ( unsigned int nRow = 0; nRow < Imp()->m_Table.size(); ++ nRow )
  {
    strContents += STR_NEWLINE;
    strContents += Imp()->m_Table[ nRow ]->AsString();
  }
  
  return strContents;
}


///////////////////////////////////////////////////////////////////////////////
bool CnlRoutingTable::Lookup( 
  CString& strGateway,
  CString& strInterface,
  const CString& strIpToLookup ) const
{
  const CnlRoutingTableRow* const pBestMatchRow = 
    Imp()->SelectBestMatchRow( strIpToLookup );

  if ( pBestMatchRow == NULL )
  {
    //2 There was no matching row!
    return false;
  }

  strGateway = pBestMatchRow->GetGateway();
  strInterface = pBestMatchRow->GetInterface();
  return true;
}

///////////////////////////////////////////////////////////////////////////////
unsigned int CnlRoutingTable::GetNicOffsetFromInterface( 
  const CString& strInterface ) const
{
  for ( ConstIpNicMapIter iter = Imp()->m_IpNicMap.begin();
        iter != Imp()->m_IpNicMap.end();
        ++iter )
  {
    const CnlIPv4Address address = ( *iter ).first;
    if ( address.Machine() == strInterface )
    {
      return ( *iter ).second;
    }
  }

  //2 Didnt' find it!
  return -1;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlRoutingTable::GetBroadcastAddressFromInterface(
  const CString& strInterface ) const
{
  unsigned int nNicOffset = GetNicOffsetFromInterface( strInterface );
  CnlIPv4Address address;
  GetIpAddress( address, nNicOffset );
  return ( address.Broadcast() );
}

///////////////////////////////////////////////////////////////////////////////
bool CnlRoutingTable::IsMyIpAddress(
  const CString& strIpAddress ) const
{
  TRACE1( _T( "\nIs IPv4 %s mine?" ), strIpAddress );

  for ( ConstIpNicMapIter iter = Imp()->m_IpNicMap.begin();
        iter != Imp()->m_IpNicMap.end();
        ++iter )
  {
    const CnlIPv4Address& address = ( * iter ).first;
    bool bIsMine = false;
    CString strReason;
    
    if ( strIpAddress == address.Machine() )
    {
      bIsMine = true;
      strReason = _T( "Exact match" );
    }
    else if ( strIpAddress == address.Broadcast() )
    {
      bIsMine = true;
      strReason = _T( "In-network broadcast" );
    }
    else if ( strIpAddress == strIpBroadcastAddress )
    {
      bIsMine = true;
      strReason = _T( "Cross-network broadcast" );
    }
    
    if ( bIsMine )
    {
      return true;
    }
  }

  return false;
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlRoutingTable::GetIpAddress(
  CnlIPv4Address& address,
  const unsigned int nNicOffset ) const
{
  ConstNicIpMapIter iter = Imp()->m_NicIpMap.find( nNicOffset );
  
  if ( iter != Imp()->m_NicIpMap.end() )
  {
    address = ( *iter ).second;

    return RESULT_OK;
  }

  return RESULT_ERROR_NIC_DOES_NOT_EXIST;
}
///////////////////////////////////////////////////////////////////////////////
void CnlRoutingTable::ClearTable()
{
  Imp()->m_Table.clear();
}
